#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/11/9



# https://blog.csdn.net/wenzhou1219/article/details/83959261
import pymysql
class QueryMetaClass(type):
    def __new__(cls, name, bases, attr_set):
        # print(85,attr_set)

        # 用於存储映射类型属性
        mapping_set = dict()

        # if name == 'QueryDB':
        #     return type.__new__(cls, name, bases, attr_set)


        # print(name)

        # for k, v in attr_dict:
        #     print(k,v)
        #     if isinstance(v, Field):
        #         print("Found Mapping:%s=>%s" % (k, v))
        #         setattr(v, 'name', k)
        #         mapping[k] = v
        #         attr_dict.pop(k)


        for k,v in attr_set.items():
            # print(k)
            # 判断是否是对应的 字段类型
            # if isinstance(v, IntegerField):
                # 将key进行设置成类属性
            setattr(v, "name", k)
            # try:
            #     mapping_set[k] = v
            # except:
            #     pass

            # if isinstance(v,AutoField):
            #     setattr(v, 'char', [k,v._restrain()])
            #     mapping_set[k] = v
                # attr_dict.pop(k)

        # 删除这些已经在字典中存储的属性
        for k in mapping_set.keys():
            attr_set.pop(k)


        # 保存属性和列的映射关系
        # attr_set['__mapping__'] = mapping_set

        # 获取用户定义的表名
        # attr_set['__table__'] = attr_set.get('Meta').table


        return type.__new__(cls, name, bases, attr_set)


class QueryDict:

    def __init__(self,kv,t):
        self.kv = kv
        self.t = t

    def __str__(self):
        try:
            return "<%s:%s>"%(self.t,self.kv["id"])
        except:
            return "<%s:%s>"%(self.t,None)

    def value(self,field):
        return self.kv[field]




class QuerySets:
    def __init__(self,kv,table):
        self.kv = kv
        self.table = table


    def value(self,field):
        return self.kv[field]

    def count(self):
        """统计当前查询的数据总行"""
        pass

    def order_by(self, **kwargs):
        """排序查询"""

    def group_by(self, **kwargs):
        """分组查询"""

    def limit(self, start, end):
        """分页"""

    def cache(self):

        for x in self.kv:
            print(x)

        return

    def __str__(self):
        # <QuerySet [<Article: Na2FdcdrEP>, <Article: skL9jdNijo>, <Article: SqwQpivvjJ>, <Article: Tivlgc2Hei>]>
        xt  = ["<Article: Na2FdcdrEP>"]
        return '<QuerySet [%s]>'


class Cache:
    def __new__(cls, *args, **kwargs):
        return


cache = Cache

class QuerySet(dict):
    def __init__(self, kv,table=None):
        # print(kv)
        self.table = table
        self.kv = kv


        super(QuerySet, self).__init__(kv)

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError("Model has not key %s" % key)

    def __setattr__(self, key, value):
        # if key == "table" or key == "kv":
        #     pass
        # else:
            self[key] = value


    def _getattribute(self,**kwargs):
        self.table = kwargs


    def data_cache(self):
        return


    def filter(self,**kwargs):
        """
        进行过滤筛选
        :param kwargs:
        :return:
        """
        fields = [] # 存放get接收的字段名称
        params = [] # 接收存放get字段的值

        if len(kwargs) > 1:
            print(len(kwargs))

        for k,v in kwargs.items():
            fields.append(k)
            params.append(str(v))

        select_dml = "select * from %s where %s=%s"%(self.table,''.join(fields),''.join(params))
        db_ct = db_connect().connect_db()
        _db_info = db_ct.cursor(cursor=pymysql.cursors.DictCursor)
        # print(_db_info)
        _db_info.execute(select_dml)

        table_fields = _db_info.description
        data = _db_info.fetchone()
        db_ct.commit()
        _db_info.close()


        setattr(cache,"data",data)
        setattr(cache,"field",kwargs)
        setattr(cache,"table",self.table)
        setattr(cache,"type",0)
        if data:
            d = QuerySet(data)
            d.pop("table")
            d.pop("kv")
            return d
        else:
            raise Exception("不存在")
        #
        # data_ = {}
        #
        # _table_fields = [x[0] for x in table_fields]
        # # print(_table_fields)
        #
        # for i,x in enumerate(_table_fields):
        #     try:
        #
        #         data_[x] = data[i]
        #     except:
        #         pass
        #
        # return self.__setattr__(i, x)


    def all(self,**kwargs):
        """获取"""
        fields = [] # 存放get接收的字段名称
        params = [] # 接收存放get字段的值

        for k, v in kwargs.items():
            fields.append(k)
            params.append(str(v))

        select_dml = "select * from %s " % self.table

        db_ct = db_connect().connect_db()
        _db_info = db_ct.cursor(cursor=pymysql.cursors.DictCursor)
        _db_info.execute(select_dml)

        data = _db_info.fetchall()

        db_ct.commit()
        _db_info.close()

        class db:
            def __init__(self,table):
                self.table = table

            def __str__(self):
                return str([QueryDict(x, self.table) for x in data])

            def count(self):
                return len(data)

            @property
            def data(self):
                r = []
                for x in data:
                    x = QuerySet(x, self.table)
                    x.pop("table")
                    r.append(x)

                return r

        return db(self.table)



    def value(self,key):
        """
        获取指定字段的值
        :param key:
        :return:
        """
        cache_ = getattr(cache,"data")
        return cache_[key]

    def delete(self):
        """
        获取指定字段的值
        :param key:
        :return:
        """
        cache_ = getattr(cache,"field")
        table = getattr(cache,"table")
        type = getattr(cache,"type")

        if type == 0:
            filed_ = []
            val = []
            for x in cache_:
                filed_.append(x)
                val.append(str(cache_[x]))

            del_db = "delete from %s where %s=%s"%(table,''.join(filed_),''.join(val))

            db_ct = db_connect().connect_db()
            _db_info = db_ct.cursor(cursor=pymysql.cursors.DictCursor)

            x = _db_info.execute(del_db)
            db_ct.commit()
            _db_info.close()
            db_ct.close()

            return x

    def get(self, **pk):
        """
        用于直接获取一条数据
        :param kwargs: 指定相应的字段与值
        :return:
        """

        fields = [] # 存放get接收的字段名称
        params = [] # 接收存放get字段的值

        for k,v in pk.items():
            fields.append(k)
            params.append(str(v))


        select_dml = "select * from %s where %s=%s"%(self.table,''.join(fields),''.join(params))
        db_ct = db_connect().connect_db()
        _db_info = db_ct.cursor(cursor=pymysql.cursors.DictCursor)

        _db_info.execute(select_dml)

        data = _db_info.fetchone()

        db_ct.commit()
        _db_info.close()

        setattr(cache,"data",data)
        setattr(cache,"field",pk)
        setattr(cache,"table",self.table)
        setattr(cache,"type",0)
        if data:
            d = QuerySet(data)
            d.pop("table")
            d.pop("kv")
            return d
        else:
            raise Exception("不存在")


from sveltest.bin.conf import settings

db_info = getattr(settings,"DATABASE")

class DBConnect(object):
    """
    数据库连接驱动
    """

    def __init__(self):
        self.host = "120.77.152.21"
        self.user = "root"
        self.database = "upload"
        self.charset = "utf8"
        self.password = "gfl13453001"


    def connect_db(self):
        """
        内置的数据库驱动连接
        :return:
        """
        return pymysql.connect(
            host=self.host,
            user=self.user,
            password=self.password,
            database=self.database,
            charset=self.charset
        )


class ModelMetaClass(type):
    def __new__(cls, name, bases, attr_set):

        # 用於存储映射类型属性
        mapping_set = dict()

        if name == 'BaseModel':
            return type.__new__(cls, name, bases, attr_set)

        for k, v in attr_set.items():
            # 判断是否是对应的 字段类型
            if isinstance(v, IntegerField):
                # 将key进行设置成类属性
                setattr(v, 'char', [k,v._restrain()])
                mapping_set[k] = v

            if isinstance(v,AutoField):
                setattr(v, 'char', [k,v._restrain()])
                mapping_set[k] = v
                # attr_dict.pop(k)

        # 删除这些已经在字典中存储的属性
        for k in mapping_set.keys():
            attr_set.pop(k)


        # 保存属性和列的映射关系
        attr_set['__mapping__'] = mapping_set

        # 获取用户定义的表名
        attr_set['__table__'] = attr_set.get('Meta').table

        return type.__new__(cls, name, bases, attr_set)



# 为了后续兼容
db_connect = DBConnect

class BaseModel(dict,metaclass=ModelMetaClass,):

    def __init__(self, **kv):

        super(BaseModel, self).__init__(**kv)

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError("Model has not key %s" % key)

    def __setattr__(self, key, value):
        self[key] = value



    def _getattribute(self):
        fields = []
        params = []
        get_fields = []

        for k, v in self.__mapping__.items():
            # print(k, v)
            # 添加对应字段的名称

            fields.append(v.char)
            get_fields.append(k)
            # 获取对应字段的参数并添加到参数列表中
            params.append(getattr(self, k, None))

        return fields,params,get_fields

    def save(self):
        val = self._getattribute()[-2:]
        print(val)

        fields = [str(x) for x in val[-1]]  # 存放get接收的字段名称
        params = [str(x) for x in val[0]]  # 接收存放get字段的值
        # print(params)
        # print(fields,','.join(fields),)

        # values = ''
        # for x in params:
        #     values += f"'{x}',"
        # print()

        # insert_into = "insert into %s (%s) values (%s)"%(self.__table__,','.join(fields),values.strip(","))
        insert_into = "insert into %s (%s) values (%s)"%(self.__table__,','.join(fields),','.join(params))
        print(insert_into)

        db_ct = db_connect().connect_db()
        _db_info = db_ct.cursor(cursor=pymysql.cursors.DictCursor)
        #
        _db_info.execute(insert_into)
        #
        # data = _db_info.fetchone()
        #
        db_ct.commit()
        _db_info.close()
        #
        # setattr(cache, "data", data)
        # setattr(cache, "field", pk)
        # setattr(cache, "table", self.table)
        # setattr(cache, "type", 0)
        # if data:
        #     d = QuerySet(data)
        #     d.pop("table")
        #     d.pop("kv")
        #     return d
        # else:
        #     raise Exception("不存在")
        #
        # print(val)
        # insert_into = f"insert into {self.__table__}  value {self} "
        # try:
        #     _db_info = db_connect().connect_db().execute(insert_into)
        #     if _db_info == 0:
        #         print("创建成功")
        # except:
        #     print("已存在")

    def create_or_update(self):
        pass

    def create_data(self):
        pass


    def drop_database(self):
        """

        :return:
        """
        drop_ddl = f"DROP TABLE {self.__table__}"

        return db_connect().connect_db().execute(drop_ddl)


    @classmethod
    def objects(self):
        """

        :return:
        """
        return QuerySet({},self.__table__)



    def create_db(self):
        """

        :return:
        """
        create_db_ddl = ""
        for i in self._getattribute()[0]:
            create_db_ddl = create_db_ddl + " ".join(i)+","

        rs_ = create_db_ddl.rstrip(",")

        exec_db = "create table %s (%s)"%(self.__table__,rs_)
        try:
            _db_info = db_connect().connect_db().execute(exec_db)
            if _db_info == 0:
                print("创建成功")
        except:
            print("已存在")


class Field(object):
    """

    """

    def __init__(self, verbose_name=None, name=None, primary_key=False,
                 max_len=None, unique=False, blank=False, null=False,
                 db_index=False, rel=None,
                 default=None,
                 editable=True,
                 serialize=True, unique_for_date=None, unique_for_month=None,
                 unique_for_year=None, choices=None, help_text='', db_column=None,
                 db_tablespace=None, auto_created=False, validators=(),
                 error_messages=None):
        """

        :param verbose_name:
        :param name:
        :param primary_key:
        :param max_len:
        :param unique:
        :param blank:
        :param null:
        :param db_index:
        :param rel:
        :param default:
        :param editable:
        :param serialize:
        :param unique_for_date:
        :param unique_for_month:
        :param unique_for_year:
        :param choices:
        :param help_text:
        :param db_column:
        :param db_tablespace:
        :param auto_created:
        :param validators:
        :param error_messages:
        """
        self.name = name
        self.verbose_name = verbose_name  # May be set by set_attributes_from_name
        self._verbose_name = verbose_name  # Store original for deconstruction
        self.primary_key = primary_key
        self.max_len, self._unique = max_len, unique
        self.blank, self.null = blank, null
        self.remote_field = rel
        self.is_relation = self.remote_field is not None
        self.default = default
        self.editable = editable
        self.serialize = serialize
        self.unique_for_date = unique_for_date
        self.unique_for_month = unique_for_month
        self.unique_for_year = unique_for_year
        # if isinstance(choices, collections.abc.Iterator):
        #     choices = list(choices)
        self.choices = choices
        self.help_text = help_text
        self.db_index = db_index
        self.db_column = db_column
        self._db_tablespace = db_tablespace
        self.auto_created = auto_created





class FieldChar(Field):


    def name(self):
        return "INT"


    def _check_parm(self):
        """

        :return:
        """


class IntegerField(Field):

    def __init__(self,null=False,default=None,unique=False):
        super(IntegerField, self).__init__(null=null,default=default,unique=unique)
        self.null = null
        self.default= default
        self.unique = unique

        # if min_value is not None:
        #     if not isinstance(min_value, numbers.Integral):
        #         raise ValueError("min_value must be int")
        #     elif min_value < 0:
        #         raise ValueError("min_value must be positive int")
        # if max_value is not None:
        #     if not isinstance(max_value, numbers.Integral):
        #         raise ValueError("max_value must be int")
        #     elif max_value < 0:
        #         raise ValueError("max_value must be positive int")
        # if min_value is not None and max_value is not None:
        #     if min_value > max_value:
        #         raise ValueError("min_value must be smaller than max_value")
        #


        self._restrain()

    # def __get__(self, instance, owner):
    #     return self._value

    # def __set__(self, instance, value):
    #     if not isinstance(value, numbers.Integral):
    #         raise ValueError("int value need")
    #     if value < self.min_value or value > self.max_value:
    #         raise ValueError("value must between min_value and max_value")
    #     self._value = value



    def _restrain(self):
        # print(self.null)
        if self.default:
            NULL = arg.DB_ARGUMENT["DEFAULT_NULL"] if self.null else f"NOT NULL DEFAULT '{self.default}'"

        elif self.unique:
            NULL = arg.DB_ARGUMENT["DEFAULT_NULL"] if self.null else f"NOT NULL  UNIQUE"
        else:

            NULL = arg.DB_ARGUMENT["DEFAULT_NULL"] if self.null else arg.DB_ARGUMENT["NOT_NULL"]
        return F"INT(10) {NULL}"



class Argument:

    DB_ARGUMENT = {
        "DEFAULT_NULL":"DEFAULT NULL",
        "NOT_NULL":"NOT NULL"
    }

arg = Argument

class AutoField(Field):

    def __init__(self,primary_key=True):
        super(AutoField, self).__init__(primary_key=primary_key)
        self._restrain()

    def _restrain(self):
        return "INT(10)  NOT NULL AUTO_INCREMENT PRIMARY KEY"

class CharField(Field):

    def __init__(self,null=False,max_len=10,default=None,unique=False):
        super(CharField, self).__init__(null=null,max_len=max_len,default=default,unique=unique)
        self._restrain()

    def _restrain(self):
        return f"varchar({self.max_len})  NOT NULL AUTO_INCREMENT PRIMARY KEY"


class UserModel(BaseModel):
    id = AutoField()
    name = CharField(default=1)
    age = IntegerField(unique=True)

    class Meta:
        table = "user"


if __name__ == '__main__':
    x = UserModel(id=3,name="中",age="4")
    x.save()

    print(x)

# INT 整型

# def name(self):
#     return 12
#
# Test2 = type("Test2",(UserModel,), {"name":name}) # 定了一个Test2类
# x = Test2()
# print(x.name())
# https://www.runoob.com/mysql/mysql-data-types.html


# import pymysql
#
# # 打开数据库连接
# db = pymysql.connect(host='localhost',
#                      user='testuser',
#                      password='test123',
#                      database='TESTDB')
#
# # 使用 cursor() 方法创建一个游标对象 cursor
# cursor = db.cursor()
#
# # 使用 execute()  方法执行 SQL 查询
# cursor.execute("SELECT VERSION()")
#
# # 使用 fetchone() 方法获取单条数据.
# data = cursor.fetchone()
#
# print("Database version : %s " % data)
#
# # 关闭数据库连接
# db.close()
#




#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/10/20

import json
import requests
from typing import Optional, List, Union, Sequence, Dict, overload, Tuple
from sveltest.bin.conf.application import app_loader
from sveltest.components._test_core import (request_env)
from sveltest.support.common import ObjectDict
from sveltest.bin.conf import settings
from sveltest.components.dblib import ShelveBase

from requests.packages.urllib3.exceptions import InsecureRequestWarning
# 禁用安全请求警告
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
# 代理池 Agent Pool

data_cache = ShelveBase()
# 内置环境控制
class RequestBase(object):

    auth_class = ObjectDict(settings.AUTH_VALIDATORS)
    environment_class = ObjectDict(settings.ENVIRONMENT_CLASSES_CONFIG)


    def __init__(self):

        self.encoding = None


    def get_auth_class(self):
        if self.auth_class.CLASS:
            cached_ = app_loader.component(self.auth_class.CLASS)
            setattr(self,"cache_",cached_)


    def get(self, router: Optional[str],
            data: Optional[Dict]=None,
            proxies:Optional[Dict]=None,
            # 对SSL证书认证
            verify:Optional[bool]=False,
            env_control:Optional[bool]=True,
            allow_redirects:Optional[bool]=False,
            timeout:Optional[Union[int,float]]=None,
            cookies:Optional[Dict]=None,
            headers:Optional[Union[Dict,bool]]=False,
            is_backend:Optional[bool]=False,
            # 客户端的证书
            cert:Optional[Union[str,Tuple[str,str]]]=None
            ):
        """

        """

        self.env_info = self.__env(self.get)
        _rh = dict(self.env_info.Header)
        __get_ = self.__get_auth_()

        try:
            if __get_:
                _rh.update(__get_)
            else:
                _rh.update(self.cache_)
        except:
            pass



        if headers is False:
            self.headers_data = _rh

        else:
            _rh.update(headers)
            self.headers_data = _rh



        if router and env_control is True:
            self.__request_module = requests.get(
                url=self._join([self.env_info.backend_env_host if is_backend  else self.env_info.frontend_env_host
                                   ,router]), params=data, proxies=proxies,
                verify=verify, allow_redirects=allow_redirects, timeout=timeout, cookies=cookies,cert=cert,
                headers=self.headers_data
            )

        else:
            self.__request_module = requests.get(
                url=router, params=data, proxies=proxies, verify=verify,allow_redirects=allow_redirects,timeout=timeout,
                cookies=cookies,headers=self.headers_data,cert=cert,
            )

        return self

    def post(self, router: Optional[str],
            data: Optional[Union[Dict,open]]=None,
            proxies:Optional[Dict]=None,
            # 对SSL证书认证
            verify:Optional[bool]=False,
            env_control:bool=True,
            is_backend: Optional[bool] = False,
            allow_redirects:Optional[bool]=False,
            timeout:Optional[Union[int,float]]=None,
            cookies:Optional[Dict]=None,
            headers:Optional[Union[Dict,bool]]=False,
            # 客户端的证书
            cert:Optional[Union[str,Tuple[str,str]]]=None
            ):
        """

        """
        self.env_info = self.__env(self.post)

        _rh = dict(self.env_info.Header)
        __get_ = self.__get_auth_()

        try:
            if __get_:
                _rh.update(__get_)
            else:
                _rh.update(self.cache_)
        except:
            pass

        if headers is False:
            self.headers_data = _rh

        else:
            _rh.update(headers)
            self.headers_data = _rh



        try:
            _req_data = json.dumps(data) if self.headers_data["Content-Type"] == "application/json" else data

        except:
            _req_data = data

        if router and env_control is True:
            self.__request_module = requests.post(
                url=self._join([self.env_info.backend_env_host if is_backend  else self.env_info.frontend_env_host
                                   ,router]), data=_req_data, proxies=proxies,
                verify=verify, allow_redirects=allow_redirects,
                timeout=timeout, cookies=cookies,cert=cert,
                headers=self.headers_data
            )

        else:
            self.__request_module = requests.post(
                url=router, data=_req_data, proxies=proxies,
                verify=verify,allow_redirects=allow_redirects,timeout=timeout,
                cookies=cookies,headers=self.headers_data,cert=cert,
            )

        return self




    def upload_file(self,file:Optional[Union[List[str],str]],type:Optional[str]="image/png"):
        """
        上传文件
        """
        if isinstance(file,str):
            with open('massive-body', 'rb') as f:
                self.post('http://some.url/streamed', data=f)

        if isinstance(file,list):

            multiple_files = [
                ('images', (x.replace("\\","/").split("/")[-1], open(x, 'rb'), 'image/png'))
                for x in file
            ]
            requests.post('url', files=multiple_files)

        return self


    @request_env()
    def put(self, router: Optional[str],
            data: Optional[Union[Dict,open]]=None,
            proxies:Optional[Dict]=None,
            # 对SSL证书认证
            verify:Optional[bool]=False,
            is_backend: Optional[bool] = False,
            env_control:bool=True,
            allow_redirects:Optional[bool]=False,
            timeout:Optional[Union[int,float]]=None,
            cookies:Optional[Dict]=None,
            headers:Optional[Union[Dict,bool]]=False,
            # 客户端的证书
            cert:Optional[Union[str,Tuple[str,str]]]=None
            ):
        """

        """

        self.env_info = self.__env(self.put)

        _rh = dict(self.env_info.Header)
        __get_ = self.__get_auth_()

        try:
            if __get_:
                _rh.update(__get_)
            else:
                _rh.update(self.cache_)
        except:
            pass

        if headers is False:
            self.headers_data = _rh

        else:
            _rh.update(headers)
            self.headers_data = _rh






        try:
            _req_data = json.dumps(data) if self.headers_data["Content-Type"] == "application/json" else data

        except:
            _req_data = data

        if router and env_control is True:
            self.__request_module = requests.put(
                url=self._join([self.env_info.backend_env_host if is_backend  else self.env_info.frontend_env_host
                                   ,router]), data=_req_data, proxies=proxies,
                verify=verify, allow_redirects=allow_redirects,
                timeout=timeout, cookies=cookies,cert=cert,
                headers=self.headers_data
            )

        else:
            self.__request_module = requests.put(
                url=router, data=_req_data, proxies=proxies,
                verify=verify,allow_redirects=allow_redirects,timeout=timeout,
                cookies=cookies,headers=self.headers_data,cert=cert,
            )

        return self

    @request_env()
    def patch(self, router: Optional[str],
            data: Optional[Union[Dict,open]]=None,
            proxies:Optional[Dict]=None,
            # 对SSL证书认证
            verify:Optional[bool]=False,
            env_control:bool=True,
            is_backend: Optional[bool] = False,
            allow_redirects:Optional[bool]=False,
            timeout:Optional[Union[int,float]]=None,
            cookies:Optional[Dict]=None,
            headers:Optional[Union[Dict,bool]]=False,
            # 客户端的证书
            cert:Optional[Union[str,Tuple[str,str]]]=None
            ):
        """

        """


        self.env_info = self.__env(self.patch)

        _rh = dict(self.env_info.Header)
        __get_ = self.__get_auth_()

        try:
            if __get_:
                _rh.update(__get_)
            else:
                _rh.update(self.cache_)
        except:
            pass


        if headers is False:
            self.headers_data = _rh

        else:
            _rh.update(headers)
            self.headers_data = _rh



        try:
            _req_data = json.dumps(data) if self.headers_data["Content-Type"] == "application/json" else data

        except:
            _req_data = data

        if router and env_control is True:
            self.__request_module = requests.patch(
                url=self._join([self.env_info.backend_env_host if is_backend  else self.env_info.frontend_env_host
                                   ,router]), data=_req_data, proxies=proxies,
                verify=verify, allow_redirects=allow_redirects,
                timeout=timeout, cookies=cookies,cert=cert,
                headers=self.headers_data
            )

        else:
            self.__request_module = requests.patch(
                url=router, data=_req_data, proxies=proxies,
                verify=verify,allow_redirects=allow_redirects,timeout=timeout,
                cookies=cookies,headers=self.headers_data,cert=cert,
            )

        return self




    @request_env()
    def delete(self, router: Optional[str],
            data: Optional[Dict]=None,
            proxies:Optional[Dict]=None,
            # 对SSL证书认证
            verify:Optional[bool]=False,
            env_control:Optional[bool]=True,
            allow_redirects:Optional[bool]=False,
            timeout:Optional[Union[int,float]]=None,
            cookies:Optional[Dict]=None,
            headers:Optional[Union[Dict,bool]]=False,
            is_backend: Optional[bool] = False,
            # 客户端的证书
            cert:Optional[Union[str,Tuple[str,str]]]=None
            ):
        """

        """

        self.env_info = self.__env(self.delete)



        _rh = dict(self.env_info.Header)
        __get_ = self.__get_auth_()

        try:
            if __get_:
                _rh.update(__get_)
            else:
                _rh.update(self.cache_)
        except:
            pass


        if headers is False:
            self.headers_data = _rh

        else:
            _rh.update(headers)
            self.headers_data = _rh


        if router and env_control is True:
            self.__request_module = requests.delete(
                url=self._join([self.env_info.backend_env_host if is_backend  else self.env_info.frontend_env_host
                                   ,router]), params=data, proxies=proxies,
                verify=verify, allow_redirects=allow_redirects, timeout=timeout, cookies=cookies,cert=cert,
                headers=self.headers_data
            )

        else:
            self.__request_module = requests.delete(
                url=router, params=data, proxies=proxies, verify=verify,allow_redirects=allow_redirects,timeout=timeout,
                cookies=cookies,headers=self.headers_data,cert=cert,
            )

        return self




    def code_ok(self):
        return self.__request_module.codes.ok


    @property
    def request_url(self):
        """

        """
        return self.__request_module.request.url

    @property
    def url(self):
        """

        """
        return self.__request_module.url



    def __get_auth_(self):
        """

        """
        if data_cache.get("token"):
            return data_cache.get("token")
        else:
            return False

    def _join(self, j:Optional[List]):
        """

        """
        return '/'.join(j)


    @property
    def content(self):
        """

        """
        return self.__request_module.content

    @property
    def text(self):
        """

        """

        return self.__request_module.text

    @property
    def json(self):
        """

        """
        return self.__request_module.json()


    def encoding_(self,coding:Optional[str]="utf-8"):
        """

        """
        self.__request_module.encoding = coding
        return self

    def save_image(self,content:Optional[bytes],fp:Optional[open]):
        """

        """
        from PIL import Image
        from io import BytesIO

        img = Image.open(BytesIO(content))
        img.save(fp=fp)
        return self

    @property
    def request_header(self):
        return HttpRequestHeaders(self.__request_module.request.headers)

    @property
    def request_headers(self):
        return self.__request_module.request.headers


    @property
    def header(self):
        """
        查看请求头
        """
        return HttpResponseHeaders(self.__request_module.headers)

    @property
    def headers(self):
        """
        查看请求头
        """
        return self.__request_module.headers


    @property
    def headers_obj(self):
        """
        查看请求头
        """
        return ObjectDict(self.__request_module.headers)

    @property
    def history(self):
        """
        查看请求头
        """
        return self.__request_module.history

    @property
    def status_code(self):
        """
        查看请求头
        """
        return self.__request_module.status_code

    def __env(self,cls_name:Optional[object]):
        """

        """
        self.cls_name = cls_name.__name__

        from sveltest.bin.conf import settings
        self.obj_conf = ObjectDict(settings.ENVIRONMENT_CLASSES_CONFIG)
        self.env = self.obj_conf.DEFAULT_ENVIRONMENT_NAME
        self.env_headers = self.obj_conf.HEADERS

        if self.env == "dev": self.ENV_CLASS = "ENVIRONMENT_DEV_HOST"
        if self.env == "sit": self.ENV_CLASS = "ENVIRONMENT_SIT_HOST"
        if self.env == "uat": self.ENV_CLASS = "ENVIRONMENT_UAT_HOST"
        if self.env == "prod": self.ENV_CLASS = "ENVIRONMENT_PROD_HOST"

        self.info = ObjectDict(self.obj_conf)
        self.env_ = ObjectDict(self.info.FRONTEND)

        # try:
        #     #:todo 暂时不进行该逻辑处理，后面版本将完善
        #     # if settings.DEBUG:
        #     #      cls.ENV_CLASS = "ENVIRONMENT_CLASS_DEV"
        #     # else:
        #     #     cls.ENV_CLASS = "ENVIRONMENT_CLASS_PROD"
        #
        #
        self.frontend_env_host = self.env_[self.ENV_CLASS]

        self.backend_env_host = self.env_[self.ENV_CLASS]
        # print( bool(cls.env_headers))
        self.Header = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36"
        }
        # print(self.Header)


        if bool(self.env_headers):
            self.Header = self.env_headers

        else:


            if self.cls_name.upper() in ["POST", "PUT", "PATCH"]:
                self.Header.update({
                    'accept': "application/json", "Content-Type": "application/json",

                })

        return ObjectDict(
            {"Header": self.Header,"frontend_env_host":self.frontend_env_host,"backend_env_host":self.backend_env_host,
             "info":self.info,"env_":self.env_}
        )

class HttpRequestHand(object):
    """用于单独使用的http请求封装API"""

    def __init__(self):

        self.encoding = None

    @request_env()
    def get(self, router: Optional[str],
            data: Optional[Dict]=None,
            proxies:Optional[Dict]=None,
            # 对SSL证书认证
            verify:Sequence[bool]=False,
            env_control:Sequence[bool]=True,
            allow_redirects:Sequence[bool]=False,
            timeout:Optional[Union[int,float]]=None,
            cookies:Optional[Dict]=None,
            headers:Optional[Union[Dict,bool]]=False,
            # 客户端的证书
            cert:Optional[Union[str,Tuple[str,str]]]=None
            ):
        """

        """

        if headers is False:
            self.headers_data = self.get.Header
        else:
            self.get.Header.update(headers)


        if router and env_control is True:
            self.__request_module = requests.get(
                url=self._join(router), params=data, proxies=proxies,
                verify=verify, allow_redirects=allow_redirects, timeout=timeout, cookies=cookies,cert=cert,
                headers=self.headers_data
            )

        else:
            self.__request_module = requests.get(
                url=router, params=data, proxies=proxies, verify=verify,allow_redirects=allow_redirects,timeout=timeout,
                cookies=cookies,headers=self.headers_data,cert=cert,
            )

        return self

    @request_env()
    def post(self, router: Optional[str],
            data: Optional[Union[Dict,open]]=None,
            proxies:Optional[Dict]=None,
            # 对SSL证书认证
            verify:Sequence[bool]=False,
            env_control:bool=True,
            allow_redirects:Sequence[bool]=False,
            timeout:Optional[Union[int,float]]=None,
            cookies:Optional[Dict]=None,
            headers:Optional[Union[Dict,bool]]=False,
            # 客户端的证书
            cert:Optional[Union[str,Tuple[str,str]]]=None
            ):
        """

        """


        try:
            _req_data = json.dumps(data) if self.headers_data["Content-Type"] == "application/json" else data

        except:
            _req_data = data

        if router and env_control is True:
            self.__request_module = requests.post(
                url=self._join(router), data=_req_data, proxies=proxies,
                verify=verify, allow_redirects=allow_redirects,
                timeout=timeout, cookies=cookies,cert=cert,
                headers=self.headers_data
            )

        else:
            self.__request_module = requests.post(
                url=router, data=_req_data, proxies=proxies,
                verify=verify,allow_redirects=allow_redirects,timeout=timeout,
                cookies=cookies,headers=self.headers_data,cert=cert,
            )

        return self




    def upload_file(self,file:Optional[Union[List[str],str]],type:Optional[str]="image/png"):
        """
        上传文件
        """
        if isinstance(file,str):
            with open('massive-body', 'rb') as f:
                self.post('http://some.url/streamed', data=f)

        if isinstance(file,list):

            multiple_files = [
                ('images', (x.replace("\\","/").split("/")[-1], open(x, 'rb'), 'image/png'))
                for x in file
            ]
            requests.post('url', files=multiple_files)

        return self

    @request_env()
    def put(self, router: Optional[str],
            data: Optional[Dict]
            ):
        """

        """

        if router:
            self.__request_module = requests.get(url=self._join(router), params=data)

            return self



    def code_ok(self):
        return self.__request_module.codes.ok

    @request_env()
    def delete(self, router: Optional[str],
            data: Optional[Dict]
               ):
        """

        """

        if router:
            self.__request_module = requests.get(url=self._join(router), params=data)

            return self

    @request_env()
    def patch(self, router: Optional[str],
            data: Optional[Dict]):
        """

        """

        if router:
            self.__request_module = requests.patch(url=self._join(router), params=data)

            return self

    @property
    def request_url(self):
        """

        """
        return self.__request_module.request.url

    @property
    def url(self):
        """

        """
        return self.__request_module.url

    @request_env()
    def _join(self, router):
        """

        """
        return '/'.join([self._join.backend_env_host, router])


    @property
    def content(self):
        """

        """
        return self.__request_module.content

    @property
    def text(self):
        """

        """

        return self.__request_module.text

    @property
    def json(self):
        """

        """
        return self.__request_module.json()


    def encoding_(self,coding:Optional[str]="utf-8"):
        """

        """
        self.__request_module.encoding = coding
        return self

    def save_image(self,content:Optional[bytes],fp:Optional[open]):
        """

        """
        from PIL import Image
        from io import BytesIO

        img = Image.open(BytesIO(content))
        img.save(fp=fp)
        return self

    @property
    def request_header(self):
        return HttpRequestHeaders(self.__request_module.request.headers)

    @property
    def request_headers(self):
        return self.__request_module.request.headers


    @property
    def header(self):
        """
        查看请求头
        """
        return HttpResponseHeaders(self.__request_module.headers)

    @property
    def headers(self):
        """
        查看请求头
        """
        return self.__request_module.headers


    @property
    def headers_obj(self):
        """
        查看请求头
        """
        return ObjectDict(self.__request_module.headers)

    @property
    def history(self):
        """
        查看请求头
        """
        return self.__request_module.history

    @property
    def status_code(self):
        """
        查看请求头
        """
        return self.__request_module.status_code

class HttpResponseHeaders:

    def __init__(self,obj:Optional[object]):
        self.obj = obj

    @property
    def content_type(self):
        """"""
        return self.obj["Content-Type"]

    @property
    def connection(self):
        """"""
        return self.obj["Connection"]

    @property
    def content_length(self):
        """"""
        return self.obj["Content-Length"]

    @property
    def keep_alive(self):
        """"""
        return self.obj["Content-Length"]

    @property
    def server(self):
        """"""
        return self.obj["Server"]

    @property
    def permissions_policy(self):
        """"""
        return self.obj["permissions-policy"]

    @property
    def last_modified(self):
        """"""
        return self.obj["Last-Modified"]

    @property
    def access_control_allow_origin(self):
        """"""
        return self.obj["Access-Control-Allow-Origin"]
    @property
    def date(self):
        """"""
        return self.obj["Date"]

    @property
    def strict_transport_security(self):
        """"""
        return self.obj["Strict-Transport-Security"]

    @property
    def etag(self):
        """"""
        return self.obj["ETag"]

    @property
    def expires(self):
        """"""
        return self.obj["expires"]

    @property
    def cache_control(self):
        """"""
        return self.obj["Cache-Control"]

    @property
    def x_proxy_cache(self):
        """"""
        return self.obj["x-proxy-cache"]

    @property
    def accept_encoding(self):
        """"""
        return self.obj["x-proxy-cache"]

class HttpRequestHeaders:

    def __init__(self,obj:Optional[object]):
        self.obj = obj

    @property
    def content_type(self):
        """"""
        return self.obj["Content-Type"]

    @property
    def connection(self):
        """"""
        return self.obj["Connection"]

    @property
    def content_length(self):
        """"""
        return self.obj["Content-Length"]

    @property
    def keep_alive(self):
        """"""
        return self.obj["Content-Length"]

    @property
    def server(self):
        """"""
        return self.obj["Server"]

    @property
    def permissions_policy(self):
        """"""
        return self.obj["permissions-policy"]

    @property
    def last_modified(self):
        """"""
        return self.obj["Last-Modified"]

    @property
    def access_control_allow_origin(self):
        """"""
        return self.obj["Access-Control-Allow-Origin"]
    @property
    def date(self):
        """"""
        return self.obj["Date"]

    @property
    def strict_transport_security(self):
        """"""
        return self.obj["Strict-Transport-Security"]

    @property
    def etag(self):
        """"""
        return self.obj["ETag"]

    @property
    def expires(self):
        """"""
        return self.obj["expires"]

    @property
    def cache_control(self):
        """"""
        return self.obj["Cache-Control"]

    @property
    def x_proxy_cache(self):
        """"""
        return self.obj["x-proxy-cache"]

    @property
    def accept_encoding(self):
        """"""
        return self.obj["x-proxy-cache"]





# https://sveltest-team.github.io/docs/logo.png

if __name__ == '__main__':

    r = RequestBase()

    data= {
    "username":13453001,"password":123456
    }
    # headers = {
    #     "Content-Type": "application/json",
    #     "accept": "application/json",
    #     "X-CSRFToken": "cRw8NvZFJPDzGSPLsie8L9qXH0w0u5ppmwIOg5cmUPREwGP6QGp8rU3E5L27R6hV",
    #     "token": "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ0b2tlbl90eXBlIjoiYWNjZXNzIiwiZXhwIjoxNjY3MzY5NTk1LCJqdGkiOiIwM2FjNjhiNzY4YzU0YWRlOGYyMzYyMTQxN2ZjYzIwMyIsInVzZXJfaWQiOiJkeldmbkhyQWFJRlV5RmJMSENSZiJ9.JwaCHh3zoG69OAZWB1_MiDPKZmKwUGHF7EvdsKY7-WM",
    # }
    # ret = r.put("http://127.0.0.1:8666/api/v1/question/12",data=data,headers=headers,env_control=False).encoding_()
    ret = r.delete("http://127.0.0.1:8666/api/v1/question/tag/keep/1471",data=json.dumps({"id":111}),env_control=False)
    # data = requests.post("http://127.0.0.1:8666/api/v1/login",headers={"data":'666'},)
    # print(data.json)
    print(r.request_headers)
    print(r.url)
    # print(data.header.connection)
    # print(data.headers)


    # print(data.status_code == data.code_ok)





#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""
                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|
"""

from collections import defaultdict
# from importlib import import_module
# from importlib.util import LazyLoader
from importlib.util import   find_spec
from  typing import List,Union,Tuple,Optional
from datetime import timedelta, datetime

from sveltest.bin.conf.appconfig import ImportString, importString
from sveltest.components.dblib import ShelveBase



# def auth_app(arg=None,**kwargs):
#     def fun(cls):
#         def wrapper(**kwargs):
#             app_loader()
#             return cls
#         return wrapper(**kwargs)
#     return fun
#


cache = ShelveBase()


class Apps:

    def __init__(self,INSERT_APP=None):
        """

        :param INSERT_APP:
        """

        self.app_insert = None

        if not INSERT_APP:
            self.app_insert = INSERT_APP

        # from sveltest.bin.conf import settings

        # self.component(settings.VALIDATORS)

        self._dict_package = defaultdict(dict)

    def check_module(self,module_name:Optional[str]) -> find_spec:
        """

        """
        module_spec = find_spec(module_name)
        if module_spec is None:
            raise ImportError("Module :{} not found".format(module_name))
        else:
            return module_spec


    # 组件
    def component(self,app_list:Union[list,tuple,str]) -> Union[list,tuple]:
        """

        """

        class_list = None
        spec_ = importString.import_string(app_list)
        if hasattr(spec_,"authenticate"):
            cur_data = spec_().authenticate()
            cache.add("token", cur_data)
            cache.quit()
            return cur_data

        return app_list

    def append(self,app_list:Union[list,tuple,str]) -> Union[list,tuple]:
        """

        """


app_loader = Apps()



# @auth_app()
class AppConfig:
    def __init__(self):
        """app加载"""



#!/usr/bin/env python
#-*- coding:utf-8 -*-


import os
from selenium import webdriver
from selenium.webdriver.support import expected_conditions as exp
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from sveltest.bin.conf import settings
from selenium.webdriver import ActionChains
import datetime
from sveltest.components.web.sume import WEBDRIVER_ELEMENT



base_url = ""

class WebDriverBase:
    """Page 封装的是公共类 所有测试类都基于该基类"""
    # Page encapsulates a common class. All test classes are based on that base class.
    def __init__(self,driver,path=None,timeout=10):
        """
        :param driver:
        :param timeout:
        :param path:


        """

        self.driver = driver
        # 加启动配置
        option = webdriver.ChromeOptions()
        option.add_argument('disable-infobars')


        if not settings.WEBDRIVER_TEST_SETTINGS:
            #全局元素定位等待
            self.driver.implicitly_wait(time_to_wait=settings.WEBDRIVER_TEST_SETTINGS["GLOBAL_IMPLICITLY_WAIT"])
        else:
            self.driver.implicitly_wait(time_to_wait=timeout)

        self.open(path)


    def option(self):
        option = webdriver.ChromeOptions()
        option.add_argument('disable-infobars')
        return option

    def open(self,path=None):
        """

        :param path:
        :return:
        """

        url = path if path else base_url

        return self.driver.get(url=url)


    def quit(self):
        """

        :return:
        """
        return self.driver.quit()






# Map PageElement constructor arguments to webdriver locator enums
LOCATOR_LIST = {
    # selenium
    'css': By.CSS_SELECTOR,
    'id_': By.ID,
    'name': By.NAME,
    'xpath': By.XPATH,
    'link_text': By.LINK_TEXT,
    'partial_link_text': By.PARTIAL_LINK_TEXT,
    'tag': By.TAG_NAME,
    'class_name': By.CLASS_NAME,

    # appium
    # 'ios_uiautomation': MobileBy.IOS_UIAUTOMATION,
    # 'ios_predicate': MobileBy.IOS_PREDICATE,
    # 'ios_class_chain': MobileBy.IOS_CLASS_CHAIN,
    # 'android_uiautomator': MobileBy.ANDROID_UIAUTOMATOR,
    # 'android_viewtag': MobileBy.ANDROID_VIEWTAG,
    # 'android_datamatcher': MobileBy.ANDROID_DATA_MATCHER,
    # 'accessibility_id': MobileBy.ACCESSIBILITY_ID,
    # 'image': MobileBy.IMAGE,
    # 'custom': MobileBy.CUSTOM,
}


class PageBaseObject(WebDriverBase):
    def __init__(self,driver,path=None,timeout=10):
        """

        :param driver:
        :param path:
        :param timeout:
        """
        super(PageBaseObject, self).__init__(driver,path=path,timeout=timeout)



    def page_(self,by,element,timeout=5,pall=0.5,describe=None):
        """

        :param by:
        :param element:
        :param timeout:
        :param pall:
        :param describe:
        :return:
        """
        describe = describe
        bys = by
        if by == by:
            return self.webWait(
                by=(WEBDRIVER_ELEMENT[by], element),
                timeout=timeout,
                pall=pall
            )
        else:
            pass

    def input(self,by,element,value,timeout=5,pall=0.5,describe=None):
        """
        :param driver:
        :param By:
        :param element:
        :param valuse:
        :param describe:
        :return:
        """
        describe = describe
        bys = by
        if by == by:
            return self.webWait(
                by=(WEBDRIVER_ELEMENT[by],element),
                timeout=timeout,
                pall=pall
            ).send_keys(value)


    def webWait(self,by, timeout=5, pall=0.5):
        """
        driver 浏览器实例对象
        timeout 等待的实际时间
        poll 刷新间隔
        :return:
        """
        try:
            element = WebDriverWait(self.driver, timeout, pall).until(
                # 直到被定位的元素被找到
                exp.presence_of_element_located(by)
            )
            return element
        except:
            print("元素定位失败,你可以尝试修改下 %s" % str(by))


    def click(self,by,element,timeout=5,pall=0.5,describe=None):
        describe = describe
        bys = by
        if by == by:
            return self.webWait(
                by=(WEBDRIVER_ELEMENT[by], element),
                timeout=timeout,
                pall=pall
            ).click()
        else:
            pass


class WebActon(PageBaseObject):

    def __init__(self,driver,path,timeout):
        super(WebActon, self).__init__(driver, path=path, timeout=timeout)
        self.page_source = self._page_source()
        self.title = self._title()
        self.url = self._current_url()

    def max_window(self):
        """

        :return:
        """
        return self.driver.maximize_window()
    def min_window(self):
        """

        :return:
        """
        return self.driver.minimize_window()

    def size_window(self, x, y):
        """

        :param x:
        :param y:
        :return:
        """
        return self.driver.set_window_size(x, y)


    def shot_save(self,path,name=None):
        try:
            if name:
                pr = os.path.join(path, name + '.png')
            else:
                sr = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
                pr = os.path.join(path, sr + '.png')
            self.driver.save_screenshot(pr)
            return True
        except Exception as e:
            return e

    # def shot_base64_save(self,path):
    #     try:
    #         sr = datetime.datetime.now().strftime("%Y-%m-%d %H-%M-%S")
    #         pr = path + '\\' + sr + '.png'
    #         self.driver.get_screenshot_as_png(pr)
    #         print('截图成功！！！')
    #     except Exception:
    #         print('截图失败！！！')

    def handles(self):
        return self.driver.window_handles

    def current_handle(self):
        return self.driver.current_window_handle

    def switch_windows(self,index):
        return self.driver.switch_to.window(index)

    def scrollTo(self,top,bottom):
        js = "window.scrollTo(%s,%s);"%(top,bottom)
        return self.driver.execute_script(js)

    def _page_source(self):
        """获取页面源代码"""
        return self.driver.page_source

    def _title(self):
        return self.driver.title

    def _current_url(self):
        return self.driver.current_url

    def select_(self,by,element,timeout=5,pall=0.5,mode=("all",None)):
        from selenium.webdriver.support.ui import Select
        if mode[0] == "all":
            if by == by:
                select_element = self.webWait(by=(WEBDRIVER_ELEMENT[by], element), timeout=timeout, pall=pall)
                all_options = select_element.find_elements_by_tag_name("option")
                return all_options
        elif mode[0] == "index":
            select = Select(self.webWait(by=(WEBDRIVER_ELEMENT[by],element),timeout=timeout,pall=pall))
            select.select_by_index(mode[1])

        elif mode[0] == "text":
            select = Select(self.webWait(by=(WEBDRIVER_ELEMENT[by],element),timeout=timeout,pall=pall))
            select.select_by_visible_text(mode[1])

        elif mode[0] == "value":
            select = Select(self.webWait(by=(WEBDRIVER_ELEMENT[by],element),timeout=timeout,pall=pall))
            select.select_by_value(mode[1])

    def deselect_all(self,by,element,timeout=5,pall=0.5):
        """取消使用选择的选项"""
        from selenium.webdriver.support.ui import Select
        select = Select(self.webWait(by=(WEBDRIVER_ELEMENT[by],element),timeout=timeout,pall=pall))
        select.deselect_all()

    def select_options(self,by,element,timeout=5,pall=0.5):
        """获取所有可选择的选项"""
        from selenium.webdriver.support.ui import Select
        select = Select(self.webWait(by=(WEBDRIVER_ELEMENT[by],element),timeout=timeout,pall=pall))
        return select.options


    def drag_and_drop(self,start,end):
        """拖拽"""

        actions = ActionChains(self.driver)
        actions.drag_and_drop(self.page_(start[0],start[1]), self.page_(end[0],end[1]))
        # 执行
        actions.perform()

    def drag_offset(self,start,x,y):
        """按照坐标 拖拽"""

        from selenium.webdriver import ActionChains
        actions = ActionChains(self.driver)
        actions.drag_and_drop_by_offset(source=self.page_(start[0],start[1]),xoffset=x,yoffset=y)
        # 执行
        actions.perform()


    def get_window_handle(self,all=False):
        """获取当前窗口句柄"""
        if all is True:
            return self.driver.window_handles
        else:
            return self.driver.current_window_handle

    def to_window(self,handle):
        """切换到指定的窗口"""
        return self.driver.switch_to.window(handle)


class PageBase(WebActon):
    """

    """
    def __init__(self,driver,path=None,timeout=5):
        """

        """
        super(PageBase, self).__init__(driver,path,timeout)










#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/9/28

"""
                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|
"""
# https://github.com/wolever/parameterized



import inspect
import json

from functools import wraps
# 数据驱动

from sveltest.components._core import Data
from sveltest.components.dblib.mysqldb import mysql_db
from sveltest.components.data.core import xml_json_loader
from sveltest.components.data.core import SvelteYaml

try:
    import yaml
except ImportError:  # pragma: no cover
    _have_yaml = False
else:
    _have_yaml = True


# 使用data
DATA_DRIVER = '%str_values'

DATA_LIST_DRIVER = '%values'

DATA_DICT_DRIVER = '%dict_data'

# json使用文件
FILE_DRIVER = '%file_path'

# 使用 yaml
YAML_LOADER_DRIVER = '%yaml_loader'

# 使用分词/解包
UNPACK_DRVER = '%unpack'

# 默认的大小写长度为5
index_len = 5




def unpack(func):
    """
    方法装饰器以添加解包特性。

    """
    setattr(func, UNPACK_DRVER, True)
    return func

def _check_data(data):
    return all(isinstance(i, (int, bool,str)) for i in data)

def _check_type(data):

    return all(isinstance(i,(tuple,list)) for i in data)

def _check_dict(data):

    return all(isinstance(i,(dict)) for i in data)

def char(*str_values):
    """
    接收数据原始数据
    """


    def wrapper(func):
        if _check_data(str_values):
            global index_len
            index_len = len(str(len(str_values)))
            setattr(func, DATA_DRIVER, str_values)
        else:
            raise Exception("不为字符串")
        return func
    return wrapper


mock_data = Data()


class MockChar:


    def __init__(self):
        pass


    @classmethod
    def enal(self,count=1,isupper=False):
        """
        英文单词
        :param mode:
        :param count:
        :param kwargs:
        :return:
        """


        if count >=1:
            data = [mock_data.alphabet(isupper=isupper) for x in range(count)]

        else:
            data = mock_data.alphabet()

        def wrapper(func):
            if _check_data(data):
                global index_len
                index_len = len(str(len(data)))
                setattr(func, DATA_DRIVER, data)
            else:
                raise Exception("不为字符串")
            return func

        return wrapper

    @classmethod
    def country(self,count=1,):
        """
        国家名称
        :param mode:
        :param count:
        :param kwargs:
        :return:
        """


        if count >=1:
            data = [mock_data.country() for x in range(count)]

        else:
            data = mock_data.country()

        def wrapper(func):
            if _check_data(data):
                global index_len
                index_len = len(str(len(data)))
                setattr(func, DATA_DRIVER, data)
            else:
                raise Exception("不为字符串")
            return func

        return wrapper

    @classmethod
    def city_suffix(self,count=1,):
        """
        国家名称
        :param mode:
        :param count:
        :param kwargs:
        :return:
        """


        if count >=1:
            data = [mock_data.city_suffix() for x in range(count)]

        else:
            data = mock_data.city_suffix()

        def wrapper(func):
            if _check_data(data):
                global index_len
                index_len = len(str(len(data)))
                setattr(func, DATA_DRIVER, data)
            else:
                raise Exception("不为字符串")
            return func

        return wrapper


    @classmethod
    def date(self,count=1,):
        """
        国家名称
        :param mode:
        :param count:
        :param kwargs:
        :return:
        """


        if count >=1:
            data = [mock_data.date() for x in range(count)]

        else:
            data = mock_data.date()

        def wrapper(func):
            if _check_data(data):
                global index_len
                index_len = len(str(len(data)))
                setattr(func, DATA_DRIVER, data)
            else:
                raise Exception("不为字符串")
            return func

        return wrapper



    def chars(self):
        pass

    def formdata(self,**kwargs):
        try:
            if kwargs["parameters"]:
                pass
        except:
            print("异常")



# def mock_char():
#     """
#     接收数据原始数据
#     """
#
#     data = ("60",6,5)
#
#
#
#     def wrapper(func):
#         if check_data(data):
#             global index_len
#             index_len = len(str(len(data)))
#             setattr(func, DATA_DRIVER, data)
#         else:
#             raise Exception("不为字符串")
#         return func
#     return wrapper
#


def extends(*values):
    """
    接收一组列表数据
    :param data:
    :return:
    """
    def wrapper(func):
        if _check_type(values):
            global index_len
            index_len = len(str(len(values)))
            setattr(func, DATA_LIST_DRIVER, values)
        else:
            raise Exception("不为列表")
        return func
    return wrapper



def serialization(*dict_data):
    """
    进行接收一组字典类型
    :param data:
    :return:
    """

    def wrapper(func):
        if _check_dict(dict_data):
            global index_len
            index_len = len(str(len(dict_data)))
            setattr(func, DATA_DICT_DRIVER, dict_data)
        else:
            raise Exception("不为字典类型")
        return func
    return wrapper


# def set_char():
#     char("1","2","3")

# def extends(data:list):
#     """
#     接收一组元组数据数据
#     :param data:
#     :return:
#     """
#     global index_len
#     index_len = len(str(len(data)))
#     return add_data(data)


class FileData:

    def __init__(self,):
        pass


    @classmethod
    def text(self,file,model=";"):
        """
        对文本文件中的内容进行读取并且进行参数化操作
        :param file:
        :param log:
        :return:
        """
        with open(file=file,mode="r",encoding="utf-8") as text:
            jdata = text.readlines()

        _data = []
        for i,v in enumerate(jdata):
            c = v.rstrip("\n")
            s = c.split(model)
            _data.append(s)

        def wrapper(func):
            if _check_type(_data):
                global index_len
                index_len = len(str(len(_data)))
                setattr(func, DATA_LIST_DRIVER, _data)
            else:
                raise Exception("不为列表")
            return func

        return wrapper


    def ini(self,file,log=";"):
        pass

    @classmethod
    def json(self,file):
        with open(file, 'r',encoding="utf-8") as f:
            json_data = json.load(f)

        def wrapper(func):
            if _check_dict(json_data):
                global index_len
                index_len = len(str(len(json_data)))
                setattr(func, DATA_DICT_DRIVER, json_data)
            else:
                raise Exception("不为json")
            return func

        return wrapper

    @classmethod
    def xml(self,file):

        xml_data = xml_json_loader(file)
        print(type(xml_data))
        # with open(file, 'r',encoding="utf-8") as f:
        #     json_data = json.load(f)

        def wrapper(func):
            # if check_dict(xml_data):
            global index_len
            index_len = len(str(len(xml_data)))
            setattr(func, DATA_DICT_DRIVER, xml_data)
            # else:
            #     raise Exception("不为json")
            return func

        return wrapper

    @classmethod
    def yaml(self,file):

        yaml_data = SvelteYaml().load(file)

        def wrapper(func):
            # if check_dict(xml_data):
            global index_len
            index_len = len(str(len(yaml_data)))
            # setattr(func, DATA_DICT_DRIVER, yaml_data)
            setattr(func, DATA_DICT_DRIVER, yaml_data)
            # else:
            #     raise Exception("不为json")
            return func

        return wrapper



def mysql(table=None):

    if table is None:
        raise  Exception("请指定需要驱动的数据库表")

    else:

        data_mysql = mysql_db().all(table=table)
        print("===data_mysql=>",list(data_mysql))

        def wrapper(func):

            global index_len
            index_len = len(str(len(data_mysql)))
            setattr(func, DATA_DICT_DRIVER, data_mysql)

            return func

        return wrapper



#
#
# a = {
#     "user_info": {
#         "id": 12,
#         "name": "Tom",
#         "age": 12,
#         "height": 160,
#         "score": 100,
#         "variance": 12
#     }
# }
#
#
# # json转xml函数
# def json_to_xml(json_str):
#     # xmltodict库的unparse()json转xml
#     # 参数pretty 是格式化xml
#     xml_str = xmltodict.unparse(json_str, pretty=1)
#     return xml_str


def generate_test_name(name, value, index=0):
    """
    生成/组装一个新的测试用例名称
    :param name: 传递的方法名
    :param value: 方法名参数
    :param index: 索引
    :return:
    """

    _index = index

    index = "{0:0{1}}".format(index + 1, index_len)

    if isinstance(value,int) or isinstance(value,str) or isinstance(value,bool):


        try:
            value = str(value)
        except UnicodeEncodeError:
            # 为兼容 python 2
            value = value.encode('ascii', 'backslashreplace')

        return "{0}_{1}".format(name, index)

    if isinstance(value,dict):
        # 数据为字典类型
        try:
            value = value["title"]  # case_name作为value值
        except:
            value = _index + 1

            # print("在使用json或dict类型时你需要在你的数据中带有 key 为title的元素")
            # raise Exception()

    # 其他
    if isinstance(value,list) or isinstance(value,tuple):
        # 如果为list则进行去元素的第一个值进行作为测试用例的名称

        test_name = "{0}_{1}_{2}".format(name, index, value[0])

        return test_name


    test_name = "{0}_{1}".format(name, value)

    return test_name


def set_testcase_data(func, newName, case_docs, *args, **kwargs):
    """
    为每个测试用例提供测试数据
    :param func:
    :param new_name:
    :param test_data_docstring:
    :param args:
    :param kwargs:
    :return:
    """

    @wraps(func)
    def wrapper(self):

        return func(self,*args, **kwargs)
    # 进行获取新的测试用例名
    wrapper.__name__ = newName



    # 被包装的测试方法
    wrapper.__wrapped__ = func
    # 如果存在被包装的方法就设置 方法的文档描述

    # 如果测试方法没有设置描述
    if case_docs is not None:
        wrapper.__doc__ = case_docs
    else:
        pass
        # 如果测试用例方法有存在描述信息
        # If the test case method has an existing description
        if func.__doc__:
            try:
                wrapper.__doc__ = func.__doc__.format(*args, **kwargs)
            except (IndexError, KeyError):
                pass

    return wrapper





def add_test(cls, test_name, test_docstring, func, *args, **kwargs):
    """
    向这个类添加一个测试用例。
    该测试将基于一个现有的功能，但将给它一个新的
    :param cls: 用例类实例
    :param test_name: 用例生成的新名称
    :param test_docstring: 用例描述
    :param func: 测试用例实例方法
    :param args: 数据参数
    :param kwargs:
    :return:
    """

    # 设置测试用例方法到当前的测试用例类中
    setattr(cls, test_name, set_testcase_data(func, test_name, test_docstring,
            *args, **kwargs))


def _add_tests_from_data(cls, name, func, data):
    """
    Add tests from data loaded from the data file into the class
    """
    for i, elem in enumerate(data):
        if isinstance(data, dict):
            key, value = elem, data[elem]
            test_name = generate_test_name(name, key, i)
        elif isinstance(data, list):
            value = elem
            test_name = generate_test_name(name, value, i)
        if isinstance(value, dict):
            add_test(cls, test_name, test_name, func, **value)
        else:
            add_test(cls, test_name, test_name, func, value)


def parameterized(arg=None, **kwargs):
    """

    :param arg:
    :param kwargs:
    :return:
    """

    def wrapper(cls):
        """

        :param cls: 拿到被装饰的类实例
        :return:
        """
        # cls.__dict__.items() 将类中的属性已列表返回可迭代对象

        for name, func in list(cls.__dict__.items()):

            # 将可迭代对象转换成列表
            # 再进行遍历分解每一个列表元素得到 方法名称及方法实例

            # 进行判断当前的方法中是否被 DATA_LIST_DRIVER 装饰
            if hasattr(func, DATA_DICT_DRIVER):

                for i, v in enumerate(getattr(func, DATA_DICT_DRIVER)):


                    test_name = generate_test_name(name, getattr(v, "__name__", v), i,)

                    test_data_docstring = func.__doc__
                    if isinstance(v, tuple) or isinstance(v, list):

                        add_test(cls, test_name, test_data_docstring, func, *v)

                    else:

                        add_test(cls, test_name, test_data_docstring, func, **v)
                delattr(cls, name)


            elif hasattr(func, DATA_LIST_DRIVER):
                for i, v in enumerate(getattr(func, DATA_LIST_DRIVER)):

                    setattr(func, UNPACK_DRVER, True)
                    test_name = generate_test_name(name, getattr(v, "__name__", v), i, )
                    test_data_docstring = func.__doc__
                    if  isinstance(v, tuple) or isinstance(v, list):
                        add_test(cls, test_name, test_data_docstring, func, *v)
                delattr(cls, name)



            elif hasattr(func, DATA_DRIVER):

                # 如果被装饰就进行对给用例数据进行枚举变量
                # getattr(func, DATA_ATTR) 首先拿到数据
                for i, v in enumerate(getattr(func, DATA_DRIVER)):
                    # 进行组装测试用例名

                    test_name = generate_test_name(name, getattr(v, "__name__", v),i,)
                    test_data_docstring = func.__doc__

                    add_test(cls,test_name,test_data_docstring,func,v)

                delattr(cls, name)
        return cls

    return wrapper(arg) if inspect.isclass(arg) else wrapper




#!/usr/bin/env python
#-*- coding:utf-8 -*-




class BasicEnvironment(object):
    """基础环境控制器"""


    def environment(self, env):
        """

        """
        raise NotImplementedError(".environment() must be overridden.")

    def environment_execute(self, env):
        """
        """
        pass


class DefaultEnvironment(BasicEnvironment):

    def environment(self, env):

        pass

    def get_environment(self):

        # x = OpenExecl(r'F:\multiplex\src\multiplex\data\test.xlsx')
        # setattr(dataSource,"DATA_SOURCE",x.get_rows_all())
        # print("xxxx====xxxx",getattr(dataSource, "DATA_SOURCE"))
        return 1

    def environment_execute(self,env):
        pass






#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time
import os
import subprocess
import sys
import requests
import zipfile
import win32api
from selenium.webdriver.chrome.options import Options
from sveltest.support.logger_v2 import log_v2
from datetime import datetime

ie = ["IE", 'Ie', 'ie']




#  xml
# https://registry.npmmirror.com/-/binary/chromedriver json
# https://registry.npmmirror.com/-/binary/geckodriver geckodriver 火狐  json
# https://msedgewebdriverstorage.blob.core.windows.net/edgewebdriver?delimiter=%2F&maxresults=100&restype=container&comp=list&_=1663815950611&timeout=60000

#  geckodriver edge  json

#  IE xml


OPEN_DERVER_URL = {
    "XML":[
        {"CHROMEDRIVER": ["https://chromedriver.storage.googleapis.com/"]},
        {"IEDRIVERSERVER": ["https://selenium-release.storage.googleapis.com/"]},
    ],
    "JSON":["https://registry.npmmirror.com/-/binary/{webdriver}"],
}





class DownloadDriver:

    # 进行解压
    def _unzip(self, to_path, zip_file):
        """
        解压Chromedriver压缩包到指定目录
        :param to_path:
        :param zip_file:
        :return:
        """
        f = zipfile.ZipFile(zip_file)
        for files in f.namelist():
            f.extract(files, to_path)

    # 配置不显示浏览器
    def head(self):
        chrome_options = Options()
        chrome_options.add_argument('--headless')
        chrome_options.add_argument('--disable-gpu')
        chrome_options.add_argument(
            'User-Agent="Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Safari/537.36')
        return chrome_options

    def timestamp(self, shijian):
        """
        :param shijian:
        :return:
        """
        s_t = time.strptime(shijian, "%Y-%m-%d %H:%M:%S")
        mkt = int(time.mktime(s_t))
        return (mkt)

    def get_driver_data(self, driver="chrome"):
        """

        :param driver:
        :return:
        """

        log_v2.info(f"当前正在搜索{driver}浏览器驱动")
        if driver in ie:

            url = OPEN_DERVER_URL["XML"][1]["IEDRIVERSERVER"][0]

        else:
            # xml
            url = OPEN_DERVER_URL["XML"][0]["CHROMEDRIVER"][0]

        log_v2.info(f"正在从 {url} 查找最新版本驱动程序...")

        _file = 'storage.xml'
        with open(_file, 'w', encoding="utf-8") as f:
            f.write(requests.get(url=url).text)
            f.close()

        try:
            import xml.etree.cElementTree as ET
        except ImportError:
            import xml.etree.ElementTree as ET

        tree = ET.parse(_file)

        # 获取根节点
        root = tree.getroot()
        if driver == "chrome":
            for x in root:
                if len(x) > 2:
                    ds = {}

                    for d in x:

                        d_ = str(d.text).split("/")
                        if d_[-1].startswith("chromedriver"):

                            ds["download"] = os.path.join(OPEN_DERVER_URL["XML"][0]["CHROMEDRIVER"][0], d.text)
                            ds["version"] = str(d.text).split("/")[0]
                        if d.tag.split("}")[-1] == "LastModified":
                            ds["updateTime"] = str(d.text).replace('T', ' ').split('.')[0]
                            ds["timestamp"] = int(self.timestamp(str(d.text).replace('T', ' ').split('.')[0]))

                    try:
                        if ds["download"]:
                            yield ds
                    except:
                        pass

        if driver in ie:
            for x in root:
                if len(x) > 2:
                    ds = {}

                    for d in x:

                        d_ = str(d.text).split("/")

                        if d_[-1].startswith("IEDriverServer"):


                            ds["download"] = os.path.join(OPEN_DERVER_URL["XML"][1]["IEDRIVERSERVER"][0], d.text)
                            ds["version"] = str(d.text).split("/")[0]

                        if d.tag.split("}")[-1] == "LastModified":
                            ds["updateTime"] = str(d.text).replace('T', ' ').split('.')[0]
                            ds["timestamp"] = int(self.timestamp(str(d.text).replace('T', ' ').split('.')[0]))


                    try:
                        if ds["download"]:
                            yield ds
                    except:
                        pass

    # 获取python安装路径
    def _get_python_path(self):
        """
        获取python安装路径
        :return:
        """
        try:
            p1 = subprocess.Popen("where python", stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode("gbk").split("\r")[0].replace("\\", "/")
        except:
            output = 0
        return output

    def _get_latest_version(self, driver="chrome"):
        """获取最新版本"""

        log_v2.info("正在搜索最新版本驱动" + driver)
        x = [x for x in self.get_driver_data(driver=driver)]

        x.sort(key=lambda s: s["timestamp"], reverse=True)


        if driver == "chrome":
            return x[:4]

        if driver in ie:
            return x[:4]

    def _get_latest_from_version(self, v, driver="chrome"):

        """根据指定的驱动版本从中进行获取最新版本驱动"""

        x = [x for x in self._get_version_driver(
            v=v,
            driver=driver
        )]
        x.sort(key=lambda x: x["timestamp"], reverse=True)
        return x[:4]

    def _get_version_driver(self, v, driver="chrome", ):
        """按照版本匹配"""
        x = [x for x in self.get_driver_data(driver=driver)]
        for s in x:
            if s["version"].startswith(v):
                yield s

    def platform(self, p="win32", driver='chrome', version=None):
        """
        version :
        :paramp: chrome:linux64 mac64 mac64_m1 win32
                ie:x64 win32
        :return:
        """


        log_v2.debug(driver)
        if driver == "chrome":
            if version:

                for x in self._get_latest_from_version(
                        v=self._get_chrome_current_version().split(".")[0],
                        driver=driver
                ):
                    if x["download"].split("/")[-1].split('.')[0].endswith(p):
                        return x

            else:
                for x in self._get_latest_version(driver=driver):
                    if x["download"].split("/")[-1].split('.')[0].endswith(p):
                        return x
        if driver in ie:

            for x in self._get_latest_version(driver=driver):

                if x["download"].split("/")[-1].split('_')[1].lower() == p:
                    try:
                        log_v2.success(
                            f'已成功找到 IEDriverServer 驱动程序版本为：{x["download"].split("/")[-1].split("_")[-1].strip("zip").strip(".").lstrip(".")}')
                    except:
                        log_v2.success(
                            '已成功找到 IEDriverServer 驱动程序')
                    return x

    def download_driver(self, path_url, filename):
        # 判断输入的文件夹或路径是否真实存在

        log_v2.debug(f"{path_url}-{filename}")

        path = "d:/drivers"
        if sys.platform[:-2] == "win":
            try:
                log_v2.info("正在创建存放驱动目录")
                os.mkdir(path)

            except:
                pass
        log_v2.success("存放驱动目录已创建成功：" + path)
        file = requests.get(url=path_url)
        save = os.path.join(path, filename).replace("\\", "/")
        # 保存文件到脚本所在目录
        with open(save, 'wb') as zip_file:
            zip_file.write(file.content)
            log_v2.success("浏览器驱动下载成功...")
            return save

    def _download(self, p="win32", driver="chrome", version=None):

        v = self.platform(p=p, driver=driver, version=version)
        log_v2.debug(v)

        if v:

            save = self.download_driver(
                path_url=v["download"],
                filename=v["download"].split("/")[-1]
            )
            log_v2.info("驱动下载路径为：" + save)
            return save
        else:
            log_v2.error("没有找到合适版本的驱动程序")

    def _unzip_driver(self, zip_path):
        log_v2.info("正在解压压缩包：" + zip_path)
        zip_path_ = self._get_python_path().split("/")
        _path = '/'.join(zip_path_[:-1])
        self._unzip(_path, zip_file=zip_path)
        log_v2.info("解压完成,解压后存放的目录为：" + _path)

    def get_chrome_current_driver_version(self):
        """

        :return:
        """
        try:
            p1 = subprocess.Popen("chromedriver -version", stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode().split(' ')
        except:
            output = 0
        return output

    def get_ie_current_driver_version(self):
        """

        :return:
        """
        try:
            p1 = subprocess.Popen("where IEDriverServer", stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode("gbk").split("\r")[0].replace("\\", "/")
            info = win32api.GetFileVersionInfo(output, os.sep)
            ms = info['FileVersionMS']
            ls = info['FileVersionLS']
            version = '%d.%d.%d.%d' % (
                win32api.HIWORD(ms), win32api.LOWORD(ms), win32api.HIWORD(ls), win32api.LOWORD(ls)
            )



        except Exception as e:
            print(e)
            version = 0
        return version

    def _get_chrome_current_version(self):
        """
        获取当前chrome浏览器
        :return:
        """
        try:
            p1 = subprocess.Popen("reg query \"HKEY_CURRENT_USER\Software\Google\Chrome\BLBeacon\" /v version",
                                  stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode().split(" ")[-1].strip()
        except:
            output = 0
        return output

    def _get_ie_current_version(self):
        """
        获取当前ie浏览器
        :return:
        """
        try:
            p1 = subprocess.Popen('reg query "HKLM\SOFTWARE\Microsoft\Internet Explorer" /v Version',
                                  stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode().split(" ")[-1].strip()
        except:
            output = 0
        return output


    def open_chrome(self, p="win32"):

        x_ = self.get_chrome_current_driver_version()
        if x_ != 0:
            log_v2.warning("已存在 chromedriver 驱动，将进行查找最新浏览器驱动...")
            dr = self.get_chrome_current_driver_version()[1]
            br = self._get_chrome_current_version()
            if int(dr.split(".")[0]) == int(br.split(".")[0]):
                log_v2.info("当前Chrome 浏览器版本为" + br + " chromedriver驱动程序版本为：" + dr)
                log_v2.info("正在查找最新驱动...")
                lv = self.platform(p=p, driver='chrome', version=br.split(".")[0])["version"]
                log_v2.info("当前符合浏览器版本的驱动最新版本为：" + lv)
                if dr == lv:
                    log_v2.warning("当前驱动程序已是最新版本，跳过下载...")
                else:
                    self._unzip_driver(zip_path=lv)
            else:
                log_v2.info("当前Chrome 浏览器版本为" + br + " chromedriver驱动程序版本为：" + dr)
                self._unzip_driver(zip_path=self._download(
                    p=p,
                    version=self._get_chrome_current_version().split(".")[0],
                ))
        else:

            self._unzip_driver(zip_path=self._download(
                p=p,
                version=self._get_chrome_current_version().split(".")[0],
            ))

    def open_ie(self, p="win32"):



        x_ = self.get_ie_current_driver_version()
        br = self._get_ie_current_version()

        if x_ != 0:
            log_v2.warning("已存在 IEDriverServer 驱动，将进行查找最新浏览器驱动...")

            dr = self.get_ie_current_driver_version()

            log_v2.debug(br)
            if int(dr.split(".")[0]) == int(br.split(".")[0]):
                log_v2.info("当前IE 浏览器版本为" + br + " IEDriverServer驱动程序版本为：" + dr)
                log_v2.info("正在查找最新驱动...")
                lv = self.platform(p=p, driver='ie', version=br.split(".")[0])["version"]
                log_v2.info("当前符合浏览器版本的驱动最新版本为：" + lv)
                if dr == lv:
                    log_v2.warning("当前驱动程序已是最新版本，跳过下载...")
                else:
                    self._unzip_driver(zip_path=lv)
            else:

                log_v2.info("当前Ie 浏览器版本为" + br + " IEDriverServer驱动程序版本为：" + dr)
                self._unzip_driver(zip_path=self._download(
                    p=p,
                    version=self._get_ie_current_version().split(".")[0],
                    driver="ie"
                )
                )
        else:
            log_v2.info("当前系统未安装有 IEDriverServer驱动程序")
            self._unzip_driver(zip_path=self._download(
                p=p,
                version=self._get_ie_current_version().split(".")[0],
                driver="ie"
            )
            )

class JsonDownloadDriver(DownloadDriver):



    def get_edge_current_version(self):
        """
        获取当前edge浏览器
        :return:
        """
        try:
            p1 = subprocess.Popen('powershell.exe "(Get-AppxPackage Microsoft.MicrosoftEdge).Version"',
                                  stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode()
        except:
            output = 0
        return output



    def _get_latest_version(self, driver="chrome"):
        """获取最新版本"""

        log_v2.info("正在搜索最新版本驱动" + driver)
        x = [x for x in self.get_driver_data(driver=driver)]
        # print()
        x.sort(key=lambda s: s["timestamp"], reverse=True)


        if driver == "chrome":
            return x[:4]

        if driver in ie:
            return x[:4]

    def _get_latest_from_version(self, v, driver="chrome"):

        """根据指定的驱动版本从中进行获取最新版本驱动"""

        x = [x for x in self._get_version_driver(
            v=v,
            driver=driver
        )]
        x.sort(key=lambda x: x["timestamp"], reverse=True)
        return x[:4]

    def _get_version_driver(self, v, driver="chrome", ):
        """按照版本匹配"""
        x = [x for x in self.get_driver_data(driver=driver)]
        for s in x:
            if s["version"].startswith(v):
                yield s

    def platform(self, p="win32", driver='chrome', version=None):
        """
        version :
        :paramp: chrome:linux64 mac64 mac64_m1 win32
                ie:x64 win32
        :return:
        """


        log_v2.debug(driver)
        if driver == "chrome":
            if version:

                for x in self._get_latest_from_version(
                        v=self._get_chrome_current_version().split(".")[0],
                        driver=driver
                ):
                    if x["download"].split("/")[-1].split('.')[0].endswith(p):
                        return x

            else:
                for x in self._get_latest_version(driver=driver):
                    if x["download"].split("/")[-1].split('.')[0].endswith(p):
                        return x
        if driver in ie:

            for x in self._get_latest_version(driver=driver):

                if x["download"].split("/")[-1].split('_')[1].lower() == p:
                    try:
                        log_v2.success(
                            f'已成功找到 IEDriverServer 驱动程序版本为：{x["download"].split("/")[-1].split("_")[-1].strip("zip").strip(".").lstrip(".")}')
                    except:
                        log_v2.success(
                            '已成功找到 IEDriverServer 驱动程序')
                    return x

    def download_driver(self, path_url, filename):
        # 判断输入的文件夹或路径是否真实存在

        log_v2.debug(f"{path_url}-{filename}")

        path = "d:/drivers"
        if sys.platform[:-2] == "win":
            try:
                log_v2.info("正在创建存放驱动目录")
                os.mkdir(path)

            except:
                pass
        log_v2.success("存放驱动目录已创建成功：" + path)
        file = requests.get(url=path_url)
        save = os.path.join(path, filename).replace("\\", "/")
        # 保存文件到脚本所在目录
        with open(save, 'wb') as zip_file:
            zip_file.write(file.content)
            log_v2.success("浏览器驱动下载成功...")
            return save

    def _download(self, p="win32", driver="chrome", version=None):

        v = self.platform(p=p, driver=driver, version=version)
        log_v2.debug(v)

        if v:

            save = self.download_driver(
                path_url=v["download"],
                filename=v["download"].split("/")[-1]
            )
            log_v2.info("驱动下载路径为：" + save)
            return save
        else:
            log_v2.error("没有找到合适版本的驱动程序")

    def _unzip_driver(self, zip_path):
        log_v2.info("正在解压压缩包：" + zip_path)
        zip_path_ = self._get_python_path().split("/")
        _path = '/'.join(zip_path_[:-1])
        self._unzip(_path, zip_file=zip_path)
        log_v2.info("解压完成,解压后存放的目录为：" + _path)

    def get_chrome_current_driver_version(self):
        """

        :return:
        """
        try:
            p1 = subprocess.Popen("chromedriver -version", stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode().split(' ')
        except:
            output = 0
        return output

    def get_ie_current_driver_version(self):
        """

        :return:
        """
        try:
            p1 = subprocess.Popen("where IEDriverServer", stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode("gbk").split("\r")[0].replace("\\", "/")
            info = win32api.GetFileVersionInfo(output, os.sep)
            ms = info['FileVersionMS']
            ls = info['FileVersionLS']
            version = '%d.%d.%d.%d' % (
                win32api.HIWORD(ms), win32api.LOWORD(ms), win32api.HIWORD(ls), win32api.LOWORD(ls)
            )



        except Exception as e:
            print(e)
            version = 0
        return version

    def _get_chrome_current_version(self):
        """
        获取当前chrome浏览器
        :return:
        """
        try:
            p1 = subprocess.Popen("reg query \"HKEY_CURRENT_USER\Software\Google\Chrome\BLBeacon\" /v version",
                                  stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode().split(" ")[-1].strip()
        except:
            output = 0
        return output

    def _get_ie_current_version(self):
        """
        获取当前ie浏览器
        :return:
        """
        try:
            p1 = subprocess.Popen('reg query "HKLM\SOFTWARE\Microsoft\Internet Explorer" /v Version',
                                  stdout=subprocess.PIPE)
            output = p1.communicate()[0].decode().split(" ")[-1].strip()
        except:
            output = 0
        return output


    def open_edge(self, p="win32"):

        x_ = self.get_chrome_current_driver_version()
        if x_ != 0:
            log_v2.warning("已存在 chromedriver 驱动，将进行查找最新浏览器驱动...")
            dr = self.get_chrome_current_driver_version()[1]
            br = self._get_chrome_current_version()
            if int(dr.split(".")[0]) == int(br.split(".")[0]):
                log_v2.info("当前Chrome 浏览器版本为" + br + " chromedriver驱动程序版本为：" + dr)
                log_v2.info("正在查找最新驱动...")
                lv = self.platform(p=p, driver='chrome', version=br.split(".")[0])["version"]
                log_v2.info("当前符合浏览器版本的驱动最新版本为：" + lv)
                if dr == lv:
                    log_v2.warning("当前驱动程序已是最新版本，跳过下载...")
                else:
                    self._unzip_driver(zip_path=lv)
            else:
                log_v2.info("当前Chrome 浏览器版本为" + br + " chromedriver驱动程序版本为：" + dr)
                self._unzip_driver(zip_path=self._download(
                    p=p,
                    version=self._get_chrome_current_version().split(".")[0],
                ))
        else:

            self._unzip_driver(zip_path=self._download(
                p=p,
                version=self._get_chrome_current_version().split(".")[0],
            ))



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/6/17
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys


WEBDRIVER_ELEMENT = {
    'css': By.CSS_SELECTOR,
    'id': By.ID,
    'name': By.NAME,
    'xpath': By.XPATH,
    'link_text': By.LINK_TEXT,
    'partial_link_text': By.PARTIAL_LINK_TEXT,
    'tag': By.TAG_NAME,
    'class_name': By.CLASS_NAME,
}


Keys_Home = Keys.HOME #按下Home键

Keys_Back_Space = Keys.BACK_SPACE #删除键（BackSpace）

Keys_Space = Keys.SPACE #空格键(Space)

Keys_Tab = Keys.TAB #制表键(Tab)

Keys_Escape =  Keys.ESCAPE #回退键（Esc）

Keys_Enter = Keys.ENTER # 回车键（Enter）

Keys_Ctrl_A = (Keys.CONTROL,'a') #全选

Keys_Ctrl_C = (Keys.CONTROL,'c') #复制（Ctrl+C）

Keys_Ctrl_X = (Keys.CONTROL,'x') #剪切（Ctrl+X）

Keys_Ctrl_V = (Keys.CONTROL,'v') #粘贴（Ctrl+V）



# send_keys(Keys.F1) 键盘F1
# ……
# Send_keys(Keys.F5)键盘F5
# …
# send_keys(Keys.F12) 键盘F12



#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""


                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|


"""
import importlib
from collections import defaultdict
from importlib import import_module
from importlib.util import LazyLoader
from importlib.util import   find_spec
from  typing import List,Union,Tuple,Optional

from sveltest.bin.conf.appconfig import ImportString, importString
from sveltest.components.network.auth import BaseAuth


class Apps:

    def __init__(self,INSERT_APP=None):
        """

        :param INSERT_APP:
        """

        self.app_insert = None
        if not INSERT_APP:
            self.app_insert = INSERT_APP



        self._dict_package = defaultdict(dict)

    def check_module(self,module_name:Optional[str]) -> find_spec:
        """

        """
        module_spec = find_spec(module_name)
        if module_spec is None:
            raise ImportError("Module :{} not found".format(module_name))
        else:
            return module_spec


    # 组件
    def component(self,app_list:Union[list,tuple,str]) -> Union[list,tuple]:
        """

        """
        class_list = None
        for x in app_list:

            spec_ = importString.import_string(x)
            if hasattr(spec_,"authenticate"):
                spec_().authenticate()


            # print(spec_import.submodule_search_locations)
            # print(import_module(x))
            # import_md = exec_module(x)

            # lazy = LazyLoader(import_md)
            # print(lazy)


            # class_list = [x for x in dir(import_md)]



            # if spec_import:
            #     exp_import = import_module(spec_import.name)
            #     print(exp_import)
            #     self._dict_package[spec_import.name] = spec_import.origin
            #     print(self._dict_package)
            #     print(spec_import.origin)
                # cls = hasattr(spec_import, "selenium")
                # print(cls)
            # else:
            #     raise

                # cls(app_name, app_module)


            # # ex = import_module(x)
            # if ex:
            #     pass
            # else:
            #     raise



        return app_list
    def append(self,app_list:Union[list,tuple,str]) -> Union[list,tuple]:
        """

        """
        class_list = None

        for x in app_list:
            print(importString.import_string(x)().authenticate(request=None))
            # print(spec_import.submodule_search_locations)
            # print(import_module(x))
            # import_md = exec_module(x)

            # lazy = LazyLoader(import_md)
            # print(lazy)


            # class_list = [x for x in dir(import_md)]



            # if spec_import:
            #     exp_import = import_module(spec_import.name)
            #     print(exp_import)
            #     self._dict_package[spec_import.name] = spec_import.origin
            #     print(self._dict_package)
            #     print(spec_import.origin)
                # cls = hasattr(spec_import, "selenium")
                # print(cls)
            # else:
            #     raise

                # cls(app_name, app_module)


            # # ex = import_module(x)
            # if ex:
            #     pass
            # else:
            #     raise



        return app_list






#!/usr/bin/env python
#-*- coding:utf-8 -*-
import os
from typing import Optional, Union, List, Dict


from faker import Faker
from sveltest.bin.conf import settings
from sveltest.support import LanguageException
from sveltest.components._api._main import VirtualData

from sveltest.components._api._email import SveltestEmail
from sveltest.support.common import ObjectDict




class Data(VirtualData):

    def __init__(self):
        super(Data,self).__init__(language=settings.DATA_MOCK_LANGUAGE)


class SvelteEmail(SveltestEmail):
    """
    内置自动化测试使用
    """

    def __init__(self):
        self.conf = ObjectDict(settings.EMAIL_CONFIG)
        super(SvelteEmail, self).__init__(user=self.conf.USERNAME,password=self.conf.PWD,host=self.conf.PORT)

    def test_send(self,attachments:Optional[List]=None):
        """

        :param attachments:
        :return:
        """
        # mail_title 标题 count 总用例数 tester 测试人员  case_run_startTime 开始运行时间 case_run_endTime 测试结束时间
        # case_run_time 运行时长 pass_count 测试通过  pass_rate 通过率 fail_count 失败用例  failure_rate 失败率
        # error_count 错误用例 error_rate 错误率 skip_count 跳过用例 skip_rate 跳过率
        jin = JinJaTemplate()
        z = jin.get_template("email_test_template01.html").render(mail_title="自动化测试", tester="测试工程师",
                                                                case_run_startTime="2022年", case_run_time="18点",case_run_endTime="1000",
                                                                count=100, pass_count=100, fail_count=100,pass_rate="100%",
                                                                error_count=100, skip_count=10, description="测试测试",
                                                                content_title="自动化测试执行结果报告",failure_rate="1%",error_rate="1%",skip_rate="1%")


        if settings.DEBUG is False:
            self.send(
                to=self.conf.LISTS,
                cc=self.conf.CC_LIST if self.conf.CC_LIST else None ,
                subject=self.conf.EMAIL_SUBJECT_PREFIX+self.conf.TITLE,
                bcc=self.conf.CC_LIST,
                contents=[str(z).replace("\n",'')],
                attachments=attachments,
            )

# NextTestRunner
class Email(SveltestEmail):
    """
    可单独使用的邮件API
    """
    def __init__(self,user:Optional[str],password:Optional[str],
                 host:Optional[str]="smtp.163.com",):
        super(Email, self).__init__(user=user,password=password,host=host)


from jinja2 import PackageLoader, Environment,FileSystemLoader


class JinJaTemplate:

    def __init__(self):
        # 创建一个包加载器对象
        self.env = Environment(loader=FileSystemLoader(os.path.join(Path(__file__).resolve().parent.parent,"bin\conf\html").replace("\\","/")))
        print(os.path.join(Path(__file__).resolve().parent.parent,"bin\conf\html"))

    def get_template(self,t:Optional[str]):
        """

        """
        return self.env.get_template(t)

# template = env.get_template('bast.html')  # 获取一个模板文件
# template.render(name='daxin', age=18)  # 渲染





if __name__ == '__main__':
    fp = r'F:\interfaceTestng\case\ocr\test_parameter_base64.py'
    from pathlib import Path
    # print(os.path.join(Path(__file__).resolve().parent.parent,"bin\conf\html").replace("\\","/"))

    j = JinJaTemplate()
    z = j.get_template("email_test_template01.html").render(html_title="自动化测试",tester="111",mail_title="自动化",
                                                            case_run_startTime="2022年",case_run_time="18点",case_run_endTime="666",
                                                            count=100,pass_count=100,fail_count=100,error_count=100,skip_count=10,description="测试测试",
                                                            content_title="自动化测试执行结果报告")


    # with open("test.html","w+",encoding="utf-8" ) as f:
    #     f.write(z)
    #     f.close()

    s = SvelteEmail()
    s.test_send()



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/6/17
import time

from selenium.webdriver.common.keys import Keys

from sveltest.components.web.base import PageBase
from sveltest.components.web.sume import (Keys_Home,Keys_Back_Space,Keys_Space,Keys_Tab,Keys_Escape,Keys_Enter,
                                      Keys_Ctrl_A,Keys_Ctrl_C,Keys_Ctrl_X,Keys_Ctrl_V)
class WebKeys(PageBase):


    # def __init__(self):
    #     super(WebKeys,self).__init__(self)

    F1 = Keys.F1
    F2 = Keys.F2
    F3 = Keys.F3
    F4 = Keys.F4
    F5 = Keys.F5
    F6 = Keys.F6
    F7 = Keys.F7
    F8 = Keys.F8
    F9 = Keys.F9
    F10 = Keys.F10
    F11 = Keys.F11
    F12 = Keys.F12


    def home(self,by,val):
        """
        :param by:
        :param val:
        :return:
        """
        return self.page_(by=by,element=val).send_keys(Keys_Home)

    def text_del(self,by,val):
        return self.page_(by=by,element=val).send_keys(Keys_Back_Space)

    def space(self,by,val,press=1):
        if press != 1:
            return self.page_(by=by,element=val).send_keys(Keys_Space * press)
            # return [self.element(by=by,ele=val).send_keys(Keys_Space) for x in range(press)]

        return self.page_(by=by,element=val).send_keys(Keys_Space)

    def tab(self,by,val):
        """
        tab键
        :param by:
        :param val:
        :return:
        """
        return self.page_(by=by,element=val).send_keys(Keys_Tab)

    def esc(self,by,val):
        """

        """
        return self.page_(by=by,element=val).send_keys(Keys_Escape)

    def enter(self,by,val):
        """

        """
        return self.page_(by=by,element=val).send_keys(Keys_Enter)

    def cla(self,by,val):
        """

        """
        return self.page_(by=by,element=val).send_keys(Keys_Ctrl_A)

    def cp(self,by,val):
        return self.page_(by=by,element=val).send_keys(Keys_Ctrl_C)

    def skr(self,by,val):
        return self.page_(by=by,element=val).send_keys(Keys_Ctrl_X)

    def ske(self,by,val):
        return self.page_(by=by,element=val).send_keys(Keys_Ctrl_V)

    def func(self,f,by=None,val=None):
        return self.page_(by=by, element=val).send_keys(f)







import json



class Cookies(PageBase):

    def get_cookies(self,url=None,path=None,timeout=20):
        self.driver.set_page_load_timeout(timeout)#限制页面加载时间
        self.driver.set_script_timeout(timeout)#限制JavaScript加载时间
        try:
            self.driver.get(url)
        except:
            self.driver.execute_script("window.stop()")
        time.sleep(5)
        input("请登录后按Enter")
        cookies = self.driver.get_cookies()
        print("cookies",cookies)
        jsonCookies = json.dumps(cookies)

        if path:
            path_file = path
        else:
            path_file = "cookies.txt"

        with open(path_file,"w",encoding="utf-8") as f:
            f.write(jsonCookies)
        time.sleep(5)
        self.driver.quit()

    def cookie_login(self,path,url=None,timeout=20):
        self.driver.set_page_load_timeout(timeout)  # 限制页面加载时间
        self.driver.set_script_timeout(timeout)  # 限制JavaScript加载时间
        try:
            self.driver.get(url)
        except:            self.driver.execute_script("window.stop()")
        f1 = open(path)
        cookies = f1.read()
        cookies = json.loads(cookies)
        for co in cookies:
            self.driver.add_cookie(co)
        self.driver.refresh()


class By(object):
    """
    Set of supported locator strategies.
    """

    ID = "id"
    XPATH = "xpath"
    LINK_TEXT = "link text"
    PARTIAL_LINK_TEXT = "partial link text"
    NAME = "name"
    TAG_NAME = "tag name"
    CLASS_NAME = "class name"
    CSS_SELECTOR = "css selector"



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/10/21
import os
import shelve
s = shelve.open('test_shelf.db')         #
#
# s['小明'] = {'int': 10, 'float': 9.5, 'String': 'Sample data'}
# s['MM'] = [1, 2, 4,"data"]
#
# print(s["小明"])
# s.close()

os.remove("./")



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/6/9
import json

import uvicorn
from fastapi import FastAPI



app = FastAPI(title="sweet 接口demo服务 ",
              description="""""",
              version="1.1")



# 请求根目录
@app.get('/')
def index():
    return {'code':0,'message': '欢迎使用 sveltest 框架 ,接口测试demo服务！'}

def get_cityids():
    data = json.load(open("cityids.json","r",encoding="utf-8"))
    data_json = {}
    for x in data:
        areaid = x["areaid"]
        countyname = x["countyname"]
        data_json[countyname] = areaid
    return data_json


def requests_weather(city_id):
    import requests
    url = f"http://aider.meizu.com/app/weather/listWeather?cityIds={city_id}"
    data = requests.get(url=url)
    dc_data = json.loads(data.content.decode("utf-8"))
    # 当前查询的城市名称
    current_city_name = dc_data["value"][0]["city"]
    # 查询的城市的省名
    current_province_name = dc_data["value"][0]["provinceName"]
    # 当前实时查询数据
    current_realtime = dc_data["value"][0]["realtime"]
    [current_realtime.pop(x) for x in ["sD","ziwaixian","img"]]
    # 城市未来六天的天气情况
    weathers = dc_data["value"][0]["weathers"]
    for i in range(len(weathers)):
        for x in ["sun_down_time", "sun_rise_time", "img", "temp_day_f", "temp_night_f","wd","ws"]:
            weathers[i].pop(x)
    # 当天的具体指标
    indexes = dc_data["value"][0]["indexes"]
    # 空气质量
    kq = dc_data["value"][0]["pm25"]
    [kq.pop(x) for x in ["advice", "citycount", "cityrank","co","color","level","no2","o3","pm10","pm25","so2","timestamp","upDateTime"]]
    data = {
        "city_id":city_id,"city_name":current_city_name,"province_name":current_province_name,
        "current_realtime":current_realtime,"weathers":weathers,"indexes":indexes,"kq":kq
    }
    return data
# 请求根目录
@app.get(
    '/weather/{cityname}',
    tags=["查询天气"],
    description="根据城市的名称进行查询当前城市的天气",
    summary="查询天气"
)
def index(cityname:str):
    """查询"""
    if cityname:
        try:
            city_id = get_cityids()[cityname]
            return requests_weather(city_id=city_id)
        except:
            return {'code':-1,'message': '没有查询到相对应的城市'}
    else:
        return {'code':1,'message': '请输入需要查询的城市名称'}


if __name__ == '__main__':
    uvicorn.run(app=app, host="127.0.0.1", port=6688)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""

                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|


"""
import datetime
import os
from datetime import timedelta

"""
    测试框架的全局常量配置及配置信息
    version: 1.0.0
"""



import platform


# BASE_DIR  = Path(__file__).resolve().parent.parent


# 框架组件注册表
APPS_LIST = [

]

# 框架控制器
VALIDATORS = []

# 认证控制器相关配置
AUTH_VALIDATORS = {
    "CLASS":"sveltest.components.network.auth.UserAuth",
    'ACCESS_TOKEN_LIFETIME': timedelta(seconds=180),  # 配置过期时间
}

# 浏览器驱动配置路径
BROWSER_DRIVER_PATH = r"D:\python3"

# 指定chrome安装路径
BINARY_LOCATION = '/opt/google/chrome/google-chrome'



# linux下的chromedriver 路径
CHROME_DRIVER_BINARY = '/usr/bin/chromedriver' # 指定chrome安装路径


if(platform.system() == 'Linux'):

    BINARY_LOCATION = '/usr/bin/chromedriver'  # 指定chrome浏览器驱动路径


# 错误重跑次数
TEST_CASE_ERROR_RETRY = 2


#错误截图
SCREENSHOTS_STATE = True


# 本地测试
DEBUG = True


# # Abnormal screenshot
# # 异常截图存放路径
SCREENSHOTS_SAVE_PATH = None

# 日志路徑
LOG_FILE_PATH = ''


# 日志状态
LOGGING_STATUS = False

# 日志输出的等级 1-3
LOGGING_VERBOSITY = 1

#测试结果
# TEST_CASE_RUN_REPORT = []


TEST_REPORT = [] # test report


# # 识别图片集
# TEST_IMG_FILE_PATH = []


default_log_file = []


# # 测试用例集路径
CASE_SUITE_PATH = None

# 测试用例收集
COLLECT_STATUS = False

# ML_HTML_REPORT_STATUS = True


# 数据库配置
DATABASE = {
    "default":{
        # 会自动进行联想python 包
        "BACKEND":"redis",
        "LOCATION":"127.0.0.1:6379",
        "OPTIONS":{
            "PASSWORD":"",
            "DB":0
        },

    }
}


###########
#  EMAIL  #
###########

# Host for sending email.
# 发送邮件的主机号
EMAIL_HOST = 'localhost'

# Port for sending email.
# 发送邮件的服务端口
EMAIL_PORT = 25


# 自动邮箱状态
AUTO_SEND_EMAIL = True

# 邮件发送人
EMAIL_CONFIG = {
    "HOST":'smtp.163.com',
    "USERNAME":"gfl13453001@163.com",
    "LISTS":["localhost@host.com"],
    "CC_LIST":[],
    "PWD":"",
    "TEMPLATE":"",
    "BACKEND":"",
    # 发送邮件成功后接收到的标题前缀
    "EMAIL_SUBJECT_PREFIX":'[sveltest] ',
    "TITLE":'自动化测试报告',
    "BCC_LISTS":[],
    "CONTENT":["测试报告"],
    "FILE":[],
}

# 测试执行环境
TEST_RUN_ENV = "sit"

# api模板服务
API_TEST_TEMPLATE = True

# web前端模拟
WEB_TEMPLATE = False


#############
#  LOGGING  #
#############

# The callable to use to configure logging
LOGGING_CONFIG = 'logging.config.dictConfig'

# Custom logging configuration.
LOGGING = {}



# 扩展程序列表
EXTENDERS_APP_LISTS = []

# 认证器默认配置
AUTHENTICATOR_CLASSES_CONFIG = {
    "CLASSES":  [
        # 默认的认证器
        "src.multiplex.core.components.auth.TokenAuth"
    ],
    "TOKEN":None
}



# 数据驱动数据源
DATA_DRIVER_SOURCE = {
    "DEFAULT_DATA_SOURCE":{
        "TYPE":"Excel",
        "PATH":None
    }
}


DEV = "dev"
DEV_HOST = "127.0.0.1"
DEV_PORT = 8000

SIT = "sit"
SIT_ENV = []
SIT_PORT = 8080

UAT = "uat"
UAT_ENV = []
UAT_PORT = 8000

PROD = "prod"
PROD_ENV = []
PROD_PORT = 80


# 环境控制器配置
ENVIRONMENT_CLASSES_CONFIG = {
    "DEFAULT_ENVIRONMENT_NAME": DEV,
    "HEADERS": {

    },
    "FRONTEND": {
        "ENVIRONMENT_DEV_HOST": "https://pypi.org",
        "ENVIRONMENT_SIT_HOST": "",
        "ENVIRONMENT_UAT_HOST": "",


    },
    "BACKEND": {
        "ENVIRONMENT_DEV_HOST": "https://pypi.org",
        "ENVIRONMENT_SIT_HOST": "",
        "ENVIRONMENT_UAT_HOST": "",
    },
}

# 后台

# 测试用例运行匹配规则
TEST_CASE_ENFORCE_RULES = "test_*.py"

# 测试用例加载标记
TEST_METHOD_PREFIX = "test"

###########
# CI      #
###########
# 持续集成相关配置
CI_START = False

# 触发规则
CI_RULE = "cron"

# cron 表达式
CI_CRON_EXPRESSION = "*"


# 数据mock 默认语言
DATA_MOCK_LANGUAGE = "zh_CN"


######################
# 钉钉群机器人通知      #
######################
# 本地测试用的钉钉群机器人配置信息
DINGDING_WEBHOOK_DEV = {
    "URL": "https://oapi.dingtalk.com/robot/send?access_token={}",
    "SECRET": "SECf204470dbbd151188c50bb903ee1a57e6396a3a3e52288af8749d0744daa50e6",
    "KEYWORD": "自动化测试报告",
}
# 钉钉发送的模板类型
DINGDING_WEBHOOK_TEMPLATE = ""

# 用于生产上的钉钉群机器人配置
DINGDING_WEBHOOK_PROD = {}


# webdriver 测试配置
WEBDRIVER_TEST_SETTINGS = {
    "GLOBAL_IMPLICITLY_WAIT":3,
    "User-Agent":3,
    "headless":False,
    "proxy-server":None,
}


#  windows toast
WINDOWS_TOAST_STATUS = True


# https://proxy.ip3366.net/user/



#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""


                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|



"""
from sveltest.components.network.main import RequestBase

class BaseAuth:
    """
    所有认证器都需要基础这个类
    """

    def authenticate(self,):
        """
        进行重写该方法来实现认证
        """
        raise NotImplementedError(".authenticate() must be overridden.")

    def authenticate_header(self,):
        """
        """
        pass


class UserAuth(BaseAuth):

    def authenticate(self):
        """必须重新该 authenticate方法"""
        ret = RequestBase()
        response_ = ret.post(router="http://127.0.0.1:8666/api/v1/login",
                             data={
                              "username": "13453001",
                              "password": "123456"
                            },
                             env_control=False)

        # 直接将整个返回值返回，但不建议怎么做
        # return response_.json
        return {"token":response_.json["token"]}



#!/usr/bin/env python
#-*- coding:utf-8 -*-


import sys
from loguru import logger



class Log4j:
    def __init__(self, level: str = "DEBUG", colorlog: bool = True):
        self.logger = logger
        self._colorlog = colorlog
        self._console_format = "[ {level} ] - <green>{time:YYYY-MM-DD HH:mm:ss}</> - {file}:{line} - <level>{message}</level>"

        self._log_format = "{time: YYYY-MM-DD HH:mm:ss} {file}  {level}  {message}"
        self._level = level
        self.level(self._colorlog, self._console_format, self._level)

    def level(self, colorlog: bool = True, format: str = None, level: str = "DEBUG",path=None):
        if format is None:
            format = self._console_format
        self.logger.remove()
        self.logger.add(sys.stderr, level=level, colorize=colorlog, format=self._console_format)
        if path:
            self.logger.add(path, level=level, colorize=False, format=self._console_format, encoding="utf-8")


# log level: TRACE < DEBUG < INFO < SUCCESS < WARNING < ERROR
log_cfg = Log4j(level="TRACE")
log_v2 = logger










#!/usr/bin/env python
# -*- coding:utf-8 -*-

# 2021/10/28

"""


                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|



sveltest

"""

import argparse
import os
import sys

from sveltest.support import StFile
from sveltest.support import ReNumber
from sveltest.bin.conf.base import BASE_DIRs

st_os = StFile()

"""
v1.0.2
guanfl
20220926
"""
from sveltest.version import version, updateTime

v = ''
if sys.platform == "win32" or sys.platform != "Linux":
    v = f"sveltest@Windows {version}  win32_x86\nUpdateTime:{updateTime}\n version {version}"
else:
    v = f"sveltest@Linux {version}  Linux32 \nUpdateTime:{updateTime}\n version {version}"

# try:
text_logo = """
                       _  _              _
                      | || |            | |
      ___ __   __ ___ | || |_  ___  ___ | |_
     / __|\ \ / // _ \| || __|/ _ \/ __|| __|
     \__ \ \ V /|  __/| || |_|  __/\__ \| |_
     |___/  \_/  \___||_| \__|\___||___/ \__|



sveltest 专注于让自动化更简洁、更简单、高效率！！！
输入 slt -h 命令你可以进行查看 sveltest cli的所有命令哦。

你也可以进行浏览器输入 sveltest 官方教程文档 ： https://sveltest-team.github.io/docs/
那么我们开始体验下吧！！！
"""

parser = argparse.ArgumentParser(
    usage="sveltest V1 cli",
    description='=' * 18 + 'sveltest-CLI' + '=' * 18,
    epilog='=' * 20 + 'sveltest' + '=' * 20)
parser.add_argument('create', help="创建工程 ", type=str, nargs='?')
# parser.add_argument('script', help="生成脚本 ",type=str,nargs='?') #可选的位置参数
parser.add_argument('run', help="运行脚本 ", type=str, nargs='?')  # 可选的位置参数
parser.add_argument('runserver', help="运行服务 ", type=str, nargs='?')  # 可选的位置参数
parser.add_argument('doc', help="浏览器打开sveltest 官方文档 ", type=str, nargs='?')  # 可选的位置参数
# ----------------------------------------------------------------

parser.add_argument('-ui', help='创建ui自动化工程项目模板 ', dest="ui")
# parser.add_argument('-api', help='创建api自动化工程项目模板 ',type=str)
# parser.add_argument('-T','--type', help='生成的脚本类型-暂指定仅为Python脚本 ')
# parser.add_argument('-file', help='源文件',type=str)
parser.add_argument('-p', '--port', help='指定端口号')
# parser.add_argument('-template', help='创建一个关键字模板',required=False)
# parser.add_argument('-path', help='脚本存放目录路径')
parser.add_argument('-run', help='运行服务')
parser.add_argument('-v', "--version", action='version', version=v, help='查看版本')

# ------------------------------------------------------------------
args = parser.parse_args()


def main():
    try:

        if args.create == "create":
            current_path = os.path.abspath("")
            # 获取用户的当前路径
            # 获取当前文件的父级目录名称
            BASE_DIR = os.path.dirname(os.path.dirname((os.path.abspath(__file__))))

            if args.ui:
                path_join = os.path.join(current_path, args.ui).replace("\\", "/")

                t = ReNumber()
                if t.number(args.ui) is False:
                    copy_fileName = os.path.join(BASE_DIRs, "project_template").replace("\\", "/")

                    # print(BASE_DIR,sys.argv[-1])

                    if os.path.isdir(path_join):
                        print(args.ui, "该目录已存在")
                        # tank_os.copy_all(copy_fileName, path_join)
                        # for wi in os.walk(path_join):
                        #     print(os.path.isdir(path_join))
                        return copy_fileName
                    else:
                        os.mkdir(args.ui)
                        # 进行复制模板到用户的指定目录下
                        st_os.copy_all(copy_fileName, path_join)

                        for rootdir, sub, current_file in os.walk(path_join):

                            # 为了兼容linux系统需要对其进行格式转换
                            cr = rootdir.replace("\\", "/").split("/")

                            # 进行遍历每一个目录下的文件
                            if cr[-1] != "__pycache__":
                                for file in current_file:
                                    # 并进行拼接他们的文件路径
                                    file_name_list = ["manage.py-tpl", "__init__.py-tpl", "BaiduElement.py-tpl"]

                                    if file in file_name_list:
                                        _join_path = os.path.join(rootdir, file).replace("\\", '/')

                                        with open(_join_path, "r", encoding="utf-8") as f:
                                            text = f.read()
                                            #
                                        apps = text.format(filename=args.ui)
                                        with open(_join_path, 'w', encoding="utf-8") as d:
                                            d.write(str(apps))
                                            d.close()
                                    # #
                                    _join_path = os.path.join(rootdir, file)
                                    _sd = _join_path.split("-")
                                    os.rename(_join_path, _sd[0])

                                if cr[-1] == "project_name":
                                    _prod = rootdir.split("\\")[:-1]
                                    join_path_string = "\\".join(_prod)
                                    dir_path = os.path.join(str(join_path_string), sys.argv[-1])
                                    os.rename(rootdir, dir_path)
                else:
                    print("ui参数值不能为数字")

            # if args.api:
            #     t = ReNumber()
            #     path_join = r"%s\%s" % (current_path, args.api)
            #     if t.number(args.api) is False:
            #         copy_fileName = os.path.join(BASE_DIR, r"config\api_project_template")
            #         # print(BASE_DIR,sys.argv[-1])
            #
            #         if os.path.isdir(path_join):
            #             print(args.api, "该目录已存在")
            #             # tank_os.copy_all(copy_fileName, path_join)
            #             # for wi in os.walk(path_join):
            #             #     print(os.path.isdir(path_join))
            #             return copy_fileName
            #         else:
            #             # print("ui",copy_fileName)
            #             os.mkdir(args.api)
            #             # 进行复制模板到用户的指定目录下
            #             st_os.copy_all(copy_fileName, path_join)
            #
            #             for rootdir, sub, current_file in os.walk(path_join):
            #
            #                 # 为了兼容linux系统需要对其进行格式转换
            #                 cr = rootdir.replace("\\", "/").split("/")
            #
            #                 # 进行遍历每一个目录下的文件
            #                 for file in current_file:
            #                     # 并进行拼接他们的文件路径
            #
            #                     file_name_list = ["manage.py-tpl", "__init__.py-tpl", "tk_test_api.py-tpl"]
            #
            #                     if file in file_name_list:
            #                         _join_path = os.path.join(rootdir, file)
            #                         # print(rootdir+"\\%s"%file)
            #                         with open(rootdir + "\\%s" % file, "r", encoding="utf-8") as f:
            #                             text = f.read()
            #                             #
            #                         apps = text.format(filename=args.api)
            #                         with open(rootdir + "\\%s" % file, 'w', encoding="utf-8") as d:
            #                             d.write(str(apps))
            #                             d.close()
            #                     # #
            #                     _join_path = os.path.join(rootdir, file)
            #                     _sd = _join_path.split("-")
            #                     os.rename(_join_path, _sd[0])
            #
            #                 if cr[-1] == "project_name":
            #                     _prod = rootdir.split("\\")[:-1]
            #                     join_path_string = "\\".join(_prod)
            #                     dir_path = os.path.join(str(join_path_string), args.api)
            #                     os.rename(rootdir, dir_path)
            #     else:
            #         print("ui参数值不能为数字")

        # 启动接口服务 仅本地地址
        elif args.create == "runserver":
            # guanfl
            # 20210610
            d = os.path.join(BASE_DIRs, 'api_demo').replace("\\", '/')

            if args.port:
                os.system(f"{d} uvicorn api:app  --port {args.port} --reload")
            else:
                os.system(fr"python {d}\api.py")

        # v 1.2.1
        elif args.create == "doc":
            os.system("start https://sveltest-team.github.io/docs/")



        # elif args.create == "script":
        #     # TODO:已完成 待测试
        #     # guanfl
        #     # 20210609
        #     # 获取用户的当前路径
        #     current_path = os.path.abspath("")
        #     path_join = None
        #
        #     if args.template == "0":
        #         BASE_DIR = os.path.dirname(os.path.dirname((os.path.abspath(__file__))))
        #         temp = os.path.join(BASE_DIR,r"default\keyword_template.xlsx")
        #         shutil.copyfile(temp, os.path.join(current_path, 'keyword_template.xlsx'))
        #
        #     if args.template != "0" and args.template is not None:
        #         BASE_DIR = os.path.dirname(os.path.dirname((os.path.abspath(__file__))))
        #         temp = os.path.join(BASE_DIR, r"default\keyword_template.xlsx")
        #         st_os.copy_all(temp, os.path.join(args.template, 'keyword_template.xlsx'))
        #
        #
        #     lab = None
        #
        #     if args.file is not None:
        #
        #         try:
        #             if os.path.exists(args.file) is False:
        #                 path_join = r"%s\%s" % (current_path, args.file)
        #             else:
        #                 path_join = args.file
        #
        #
        #             script_path = TankOpenExecl(file=path_join)
        #             # 获取表单的列和行数
        #             get_col_row = [script_path.max_row, script_path.max_col]
        #
        #             for d in range(len(script_path.get_sheets())):
        #                 # 进行限制没有进行修改工作表名将进行不出来代码生成
        #                 if script_path.get_sheets()[d][0:-1] != "Sheet" and script_path.get_sheets()[d] != "示例":
        #                     # 存储所有数据
        #                     data_all = []
        #
        #                     script_path.set_active(d)
        #
        #                     for c in range(0, get_col_row[0]):
        #                         # 存储每一行的数据
        #                         cs = []
        #                         for r in range(get_col_row[1]):
        #                             cs.append(script_path.cell(c + 1, r + 1))
        #                         # 每一行数据进行存储到所有数据列表中
        #                         data_all.append(cs)
        #
        #                     func_list = []
        #
        #                     save_file_path = None
        #                     if args.path:
        #                         save_file_path = os.path.join(args.path,f'{script_path.filename[d]}.py')
        #                     else:
        #                         save_file_path = f'{script_path.filename[d]}.py'
        #
        #                     with open(save_file_path, "w+", encoding="utf-8") as f:
        #
        #                         for s in range(len(data_all)):
        #                             if s != 0:
        #                                 if data_all[s][0] == "url":
        #                                     f.write("from selenium import webdriver\n")
        #                                     f.write("from selenium.webdriver.common.by import By\n")
        #                                     f.write("import time\n\n\n")
        #
        #                                     f.write(f"class {script_path.filename[d]}:\n\n")
        #                                     f.write(f"    def __init__(self):\n")
        #                                     url = str(data_all[s][1])
        #                                     try:
        #                                         note = data_all[s][2]
        #                                         if note: f.write(f"        #{note}\n")
        #                                     except:
        #                                         pass
        #                                     f.write(f"        self.url = '{url}'\n")
        #                                 if data_all[s][0] == "open":
        #                                     driver = data_all[s][1]
        #                                     try:
        #                                         note = data_all[s][2]
        #                                         if note: f.write(f"        #{note}\n")
        #                                     except:
        #                                         pass
        #
        #                                     if driver == "chrome" or driver == "Chrome":
        #                                         driver = "Chrome"
        #                                         f.write(f"        self.driver = webdriver.{driver}()\n")
        #
        #                                     if driver == "ie" or driver == "IE":
        #                                         f.write(f"        self.driver = webdriver.Ie()\n")
        #                                     f.write(f"        self.driver.get(self.url)\n\n")
        #
        #                                 if data_all[s][0] == "func":
        #                                     func_list.append(data_all[s][1])
        #                                     f.write(f"    def test_{data_all[s][1]}(self):\n")
        #
        #                                 if data_all[s][0] == "sleep":
        #                                     try:
        #                                         note = data_all[s][2]
        #                                         if note: f.write(f"        #{note}\n")
        #                                     except:
        #                                         pass
        #                                     f.write(f"        time.sleep(4)\n")
        #
        #                                 if data_all[s][0] == "input":
        #
        #                                     try:
        #                                         note = data_all[s][3]
        #                                         if note: f.write(f"        #{note}\n")
        #                                     except:
        #                                         pass
        #                                         # upper() #转换成大写
        #                                     element_by = str(data_all[s][1]).split("=")
        #                                     f.write(
        #                                         f"        self.driver.find_element(by=By.{element_by[0].upper()},value='{element_by[1]}').send_keys('{data_all[s][2]}')\n")
        #
        #                                 if data_all[s][0] == "end":
        #
        #                                     try:
        #                                         note = data_all[s][3]
        #                                         if note: f.write(f"        #{note}\n")
        #                                     except:
        #                                         pass
        #                                     try:
        #                                         f.write(
        #                                             f"\n\n\nif __name__ == '__main__':\n    case = {script_path.filename[d]}()\n    case.test_{func_list[0]}()\n")
        #                                     except:
        #                                         print("不允许在没有声明方法前进行使用end关键字")
        #                                     break
        #         except:
        #             print("你必须进行指定一个Excel文件或者绝对路径")

        elif args.create == "run":
            # 运行测试脚本
            current_path = os.path.abspath("").replace('\\', '/')
            os.system("python manage.py")



        else:
            print("没有这个命令：" + sys.argv[1])
    except:
        print(text_logo)


if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        pass






#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""
                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|
"""

import yaml


#
class SvelteYaml:
    """

    """
    def load(self,file):
        dt = open(file, "r", encoding="utf-8")
        return yaml.load(dt.read(), Loader=yaml.FullLoader)

import configparser
config = configparser.ConfigParser() # 类实例化




class TypeDict(dict):

    def __init__(self, **kv):

        super(TypeDict, self).__init__(**kv)

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError("Model has not key %s" % key)

    def __setattr__(self, key, value):
        self[key] = value




class DictPlus(dict):
    """

    """
    def __init__(self, d=None):
        if d is not None:
            for k,v in d.items():
                self[k] = v
        super().__init__()

    def __key(self, key):
        return "" if key is None else key.lower()

    def __str__(self):
        import json
        return json.dumps(self)

    def __setattr__(self, key, value):
        self[self.__key(key)] = value

    def __getattr__(self, key):
        return self.get(self.__key(key))

    def __getitem__(self, key):
        return super().get(self.__key(key))

    def __setitem__(self, key, value):
        return super().__setitem__(self.__key(key), value)


class EasyDict(dict):
    """
    Get attributes

    >>> d = EasyDict({'foo':3})
    >>> d['foo']
    3
    >>> d.foo
    3
    >>> d.bar
    Traceback (most recent call last):
    ...
    AttributeError: 'EasyDict' object has no attribute 'bar'

    Works recursively

    >>> d = EasyDict({'foo':3, 'bar':{'x':1, 'y':2}})
    >>> isinstance(d.bar, dict)
    True
    >>> d.bar.x
    1

    Bullet-proof

    >>> EasyDict({})
    {}
    >>> EasyDict(d={})
    {}
    >>> EasyDict(None)
    {}
    >>> d = {'a': 1}
    >>> EasyDict(**d)
    {'a': 1}

    Set attributes

    >>> d = EasyDict()
    >>> d.foo = 3
    >>> d.foo
    3
    >>> d.bar = {'prop': 'value'}
    >>> d.bar.prop
    'value'
    >>> d
    {'foo': 3, 'bar': {'prop': 'value'}}
    >>> d.bar.prop = 'newer'
    >>> d.bar.prop
    'newer'


    Values extraction

    >>> d = EasyDict({'foo':0, 'bar':[{'x':1, 'y':2}, {'x':3, 'y':4}]})
    >>> isinstance(d.bar, list)
    True
    >>> from operator import attrgetter
    >>> map(attrgetter('x'), d.bar)
    [1, 3]
    >>> map(attrgetter('y'), d.bar)
    [2, 4]
    >>> d = EasyDict()
    >>> d.keys()
    []
    >>> d = EasyDict(foo=3, bar=dict(x=1, y=2))
    >>> d.foo
    3
    >>> d.bar.x
    1

    Still like a dict though

    >>> o = EasyDict({'clean':True})
    >>> o.items()
    [('clean', True)]

    And like a class

    >>> class Flower(EasyDict):
    ...     power = 1
    ...
    >>> f = Flower()
    >>> f.power
    1
    >>> f = Flower({'height': 12})
    >>> f.height
    12
    >>> f['power']
    1
    >>> sorted(f.keys())
    ['height', 'power']

    update and pop items
    >>> d = EasyDict(a=1, b='2')
    >>> e = EasyDict(c=3.0, a=9.0)
    >>> d.update(e)
    >>> d.c
    3.0
    >>> d['c']
    3.0
    >>> d.get('c')
    3.0
    >>> d.update(a=4, b=4)
    >>> d.b
    4
    >>> d.pop('a')
    4
    >>> d.a
    Traceback (most recent call last):
    ...
    AttributeError: 'EasyDict' object has no attribute 'a'
    """
    def __init__(self, d=None, **kwargs):
        if d is None:
            d = {}
        if kwargs:
            d.update(**kwargs)
        for k, v in d.items():
            setattr(self, k, v)
        # Class attributes
        for k in self.__class__.__dict__.keys():
            if not (k.startswith('__') and k.endswith('__')) and not k in ('update', 'pop'):
                setattr(self, k, getattr(self, k))

    def __setattr__(self, name, value):
        if isinstance(value, (list, tuple)):
            value = [self.__class__(x)
                     if isinstance(x, dict) else x for x in value]
        elif isinstance(value, dict) and not isinstance(value, self.__class__):
            value = self.__class__(value)
        super(EasyDict, self).__setattr__(name, value)
        super(EasyDict, self).__setitem__(name, value)

    __setitem__ = __setattr__

    def update(self, e=None, **f):
        d = e or dict()
        d.update(f)
        for k in d:
            setattr(self, k, d[k])

    def pop(self, k, d=None):
        delattr(self, k)
        return super(EasyDict, self).pop(k, d)




# TODO :已完成 待测试
# v1.0
# guanfl
# 20210528
class TkJson:
    def __init__(self):
        pass
    #转化字典
    def dict(self,data):
        return json.loads(data)

    #读取json文件中转化字典
    def json_file(self,file):
        jsObj = json.load(open(file,encoding="utf-8"))
        return jsObj

    #转换成json字符串
    def tkjson(self,data):
        return json.dumps(data)


    #将python数据类型文件转换成json文件数据
    def dict_json_file(self,data,file):
        with open(file,"w",encoding="utf-8") as f:
            return json.dump(data,f,indent=2)

    #不以ascii编码
    def json_cn(self,data):
        return json.dumps(data,ensure_ascii=False,indent=2)

    #不以ascii编码 美化json格式 indent 子元素每个空格2格
    def json_cn_format(self,data):
        return json.dumps(data,ensure_ascii=False,indent=2)


    #写json文件
    def tk_json_file(self,file,data):
        with open(file,"w",encoding="utf-8") as f:
            f.write(self.json_cn_format(data=data))

import json
import xmltodict

# 定义xml转json的函数
def xml_to_json(xml_str):
    # parse是的xml解析器
    xml_parse = xmltodict.parse(xml_str)
    # json库dumps()是将dict转化成json格式,loads()是将json转化成dict格式。
    # dumps()方法的ident=1,格式化json
    json_str = json.dumps(xml_parse, indent=1)
    dict_json = json.loads(json_str)

    return dict_json

def xml_json_loader(file):
    with open(file,"r",encoding="utf-8") as f:
        xmk_ = f.read()
        f.close()
    data = xml_to_json(xmk_)
    return data



#!/usr/bin/env python
#-*- coding:utf-8 -*-

import importlib
import os

from selenium import webdriver


import unittest

# class GetSettings:
#
#     def __init__(self):
#         os.environ.setdefault('settings', 'sweet.lib.unit.common.settings')
#
#     def setting_api(self):
#         from sweet.config.conf import settings
#         from easydict import EasyDict as edict
#         return edict(getattr(settings,"MULTIPLEX_CONFIG"))
#
# api_settings = GetSettings().setting_api()

# 扩展程序列表
class TestCaseView(unittest.TestCase):
    # Environment controller
    # 环境控制器
    # environment_classes = None
    # Assertion validator
    # 断言验证器
    # assertion_validator_classes = None
    # Test executor
    # 测试执行器
    # test_executor_classes = None
    # Logger
    # 日志器
    # logger_classes = None
    # Version controller
    # 版本控制器
    # version_controller_classes = None
    # Session authenticator
    # 认证器
    # authenticator_classes = api_settings.AUTHENTICATOR_CLASSES.CLASSES
    # Data drive
    # 数据驱动器
    #:TODO 暂未完成的功能
    # datadriver_classes = api_settings.DATA_DRIVER_CLASSES

    def __init__(self,methodName='runTest'):
        """

        :param methodName:
        """
        super(TestCaseView, self).__init__(methodName=methodName)
        self.data_driver = None



    # def beforeCreate(self):
    #     print("6666")
    #     self._run_datadriver()

    # def created(self):
    #
    #     self._run_authenticator()


    # def get_data(self):
    #     from sweet.config.conf import settings
    #
    #     return getattr(settings,"TEST_DATA")

    # def get_testview_description(self):
    #     """
    #     获取测试用例的方法描述
    #     :return:
    #     """
    #
    #
    # def get_authenticators(self):
    #     """
    #     Instantiates and returns the list of authenticators that this view can use.
    #     """
    #     return [auth() for auth in self.authenticator_classes]
    #
    # def _run_authenticator(self):
    #     """
    #
    #     :return:
    #     """
    #     # set_auth = self.authenticator_classes
    #     if isinstance(self.authenticator_classes,list):
    #         try:
    #             set_auth = self.get_authenticators()
    #             if len(set_auth) >= 1:
    #                 self.auth = {"token":2}
    #                 self.get_authenticators()[0].authenticate(self.auth)
    #         except:
    #             # 废弃
    #             # module = importlib.import_module(self.authenticator_classes[0])
    #             module = ImportString().import_string(self.authenticator_classes[0])
    #             module().authenticate_header(True)

    # def get_datadrivers(self):
    #     """
    #
    #     :return:
    #     """
    #     return [data() for data in self.datadriver_classes]
    #
    #
    # def _run_datadriver(self):
    #     """
    #
    #     :return:
    #     """
    #     if isinstance(self.datadriver_classes,list):
    #         try:
    #             set_data = self.get_datadrivers()
    #             if len(set_data) >= 1:
    #                     self.get_datadrivers()[0].datadrvier(data=1)
    #
    #         except:
    #             # 废弃
    #             # module = importlib.import_module(self.authenticator_classes[0])
    #             module = ImportString().import_string(self.datadriver_classes[0])
    #             print("=============>",module)
    #             module().datadrvier_execute(True)
    #
    #


# class APITestCaseView(TestCaseView,Client):
#     """
#     """
#     def __init__(self,methodName='runTest'):
#         super(APITestCaseView, self).__init__(methodName=methodName)
#         self.header_token = False
#         self._get_public = None
#         self._token = None
#         self.headers = {}
#         self._get_auth_config()
#


class UITestCaseView(TestCaseView):
    def __init__(self,methodName='runTest'):
        super(UITestCaseView, self).__init__(methodName=methodName)




    # def chrome(self):
    #     driver = webdriver.Chrome()
    #     self.driver = pageElement(driver)
    #     return self.driver
    #
    # def open(self,path=None):
    #     self.chrome().open(path)



# class APPTestView(TestView,Client):
#     def __init__(self,methodName='runTest'):
#         super(APPTestView, self).__init__(methodName=methodName)
#
# class QTTestView(TestView,Client):
#     def __init__(self,methodName='runTest'):
#         super(QTTestView, self).__init__(methodName=methodName)
#
# class GenericTestView(APITestView,UITestView,APPTestView,QTTestView):
#     def __init__(self,methodName='runTest'):
#         super(GenericTestView, self).__init__(methodName=methodName)






#!/usr/bin/env python
#-*- coding:utf-8 -*-

# 2021/12/10


import pymysql
from sveltest.components.dblib.sql import DBConnect


class DBConnectMysSQL(DBConnect):
    """

    """
    def all(self,table):
        sql = "SELECT * FROM %s"%table
        cursor_data = self.connect_db().cursor(cursor=pymysql.cursors.DictCursor)
        cursor_data.execute(sql)
        data = cursor_data.fetchall()

        self.connect_db().commit()
        self.connect_db().close()
        return data

    def get(self,table,pk):
        sql = "SELECT * FROM %s WHERE id=%s"%(table,pk)
        cursor_data = self.connect_db().cursor(cursor=pymysql.cursors.DictCursor)
        cursor_data.execute(sql)
        data = cursor_data.fetchall()

        self.connect_db().commit()
        self.connect_db().close()
        return data

mysql_db = DBConnectMysSQL



if __name__ == '__main__':
    db = DBConnectMysSQL()
    x = db.get("user",'1')
    print(x)
